--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]] 

-- define addon name
local addonName = "DKPLootMaster";
local addonVersion = "1.1.1";
local addonShortcut = "|cffDA8CFF[DLM]|r ";

-- define addon
DkpLootMasterAddon = {};
DkpLootMasterAddon.__index = DkpLootMasterAddon;

-- prepare logs
if (Logs == nil) then
    Logs = {}
end

--- Create new addon container.
function DkpLootMasterAddon:Create()
    -- create addon and add empty holders
    local addon = {
        name = addonName,
        version = addonVersion,
        shortcut = addonShortcut,
        debug = false,
        trace = false,
        frame = CreateFrame("Frame"),
        logLevel = 0,
        loaded = false
    };
    setmetatable(addon, DkpLootMasterAddon);
    
    -- clear types
    addon.serviceTypes = {};
    addon.services = {};
    addon.viewTypes = {};
    addon.modelTypes = {};

    -- register events
    addon:RegisterEvents();
    return addon;
end

--- Register service.
-- @param ServiceType Service type definition.
-- @param name Name of service.
function DkpLootMasterAddon:RegisterService(ServiceType, name)
    -- add to service types
    self.serviceTypes[name] = ServiceType;
end

--- Register view.
-- @param ViewType View type definition.
-- @param name Name of view.
function DkpLootMasterAddon:RegisterView(ViewType, name)
    -- add to view types
    self.viewTypes[name] = ViewType;
end

--- Register model.
-- @param ModelType Model type definition.
-- @param name Name of model.
function DkpLootMasterAddon:RegisterModel(ModelType, name)
    -- add to model types
    self.modelTypes[name] = ModelType;
end

--- Get service.
-- @param name Name of service.
function DkpLootMasterAddon:GetService(name)
    -- check if service created
    if (self.services[name] == nil) then
        -- create service
        local service = {};
        setmetatable(service, self.serviceTypes[name]);
        self.services[name] = service;

        -- initialize service
        service:Initialize();
    end

    -- get service
    return self.services[name];
end

--- Create view.
-- @param name Name of view.
function DkpLootMasterAddon:CreateView(name)
    -- get to view type
    local viewType = self.viewTypes[name];

    -- create view
    local view = {};
    setmetatable(view, viewType);
    return view;
end

--- Get model.
-- @param name Name of model.
function DkpLootMasterAddon:GetModel(name)
    -- get model type
    return self.modelTypes[name];
end

--- Toggle debug flag.
function DkpLootMasterAddon:ToggleDebug()
    if (self.logLevel == 1) then
        self.logLevel = 0;
        self:GetService("chat"):WriteBare("Debug-Mode disabled.");
    elseif (self.logLevel == 0) then
        self.logLevel = 1;
        self:GetService("chat"):WriteBare("Debug-Mode enabled.");
    end
end

--- Toggle trace flag.
function DkpLootMasterAddon:ToggleTrace()
    if (self.logLevel == 2) then
        self.logLevel = 0;
        self:GetService("chat"):WriteBare("Trace-Mode disabled.");
    elseif (self.logLevel == 0) then
        self.logLevel = 2;
        self:GetService("chat"):WriteBare("Trace-Mode enabled.");
    end
end

--- Log debug message.
-- @param class Name of class to log within.
-- @param method Name of method to log within.
-- @param message Log message.
function DkpLootMasterAddon:LogDebug(class, method, message, ...)
    -- check if trace not enabled
    if (self.logLevel < 1) then
        return;
    end

    -- add to logs
    table.insert(Logs, date("%Y-%m-%dT%H-%M-%S") .. " " .. class .. ":" .. method .. " [Debug] " .. string.format(message, ...));

    -- print out trace
    print("[Debug] " .. class .. ":" .. method .. " - " .. string.format(message, ...));
end

--- Log trace message.
-- @param class Name of class to log within.
-- @param method Name of method to log within.
-- @param message Log message.
function DkpLootMasterAddon:LogTrace(class, method, message, ...)
    -- check if trace not enabled
    if (self.logLevel < 2) then
        return;
    end

    -- add to logs
    table.insert(Logs, date("%Y-%m-%dT%H-%M-%S") .. " " .. class .. ":" .. method .. " [Trace] " .. string.format(message, ...));

    -- print out trace
    print("[Trace] " .. class .. ":" .. method .. " - " .. string.format(message, ...));
end

--- Register addon events.
function DkpLootMasterAddon:RegisterEvents()
    -- register events
    self.frame:RegisterEvent("PLAYER_LOGIN");
    self.frame:RegisterEvent("PLAYER_ENTERING_WORLD");
    self.frame:RegisterEvent("PARTY_LOOT_METHOD_CHANGED");
    self.frame:RegisterEvent("RAID_INSTANCE_WELCOME");
    self.frame:RegisterEvent("GROUP_LEFT");
    self.frame:RegisterEvent("PARTY_LEADER_CHANGED");
    self.frame:RegisterEvent("GROUP_ROSTER_UPDATE");
    self.frame:RegisterEvent("CHAT_MSG_ADDON");
    self.frame:RegisterEvent("LOOT_READY");
    self.frame:RegisterEvent("BOSS_KILL");

    -- handle on event
    self.frame:SetScript("OnEvent", function(_self, event, prefix, message, channel, sender)
        -- handle chat messaage
        if (event == "CHAT_MSG_ADDON") then
            -- startup
            self:GetService("message"):HandleMessage(prefix, message, sender);

        -- check if group changed
        elseif (event == "GROUP_ROSTER_UPDATE") then
            self:GetService("timer"):Wait("PlayersUpdated", 2, function()
                self:GetService("group"):PlayersUpdated();
            end);

        -- handle party changed / instance joined / loot cahnged events
        elseif (event == "PARTY_LOOT_METHOD_CHANGED" or event == "RAID_INSTANCE_WELCOME" or event == "PARTY_LEADER_CHANGED") then
            self:GetService("timer"):Wait("MasterLooterCheck", 2, function()
                self:GetService("queue"):CheckLootWatch();
            end);

        -- handle loot ready
        elseif (event == "LOOT_READY") then
            self:GetService("queue"):CheckCurrentLoot();

        -- handle group left event
        elseif (event == "GROUP_LEFT") then
            self:GetService("queue"):CheckLootWatch();

        -- handle encounter end
        elseif (event == "BOSS_KILL") then
            self:GetService("queue"):BossKilled(message);

        -- handle enter world
        elseif (event == "PLAYER_ENTERING_WORLD") then
            -- startup
            self:GetService("timer"):Wait("MasterLooterCheck", 2, function()
                self:GetService("queue"):CheckLootWatch();
            end);
            
        -- handle login
        elseif (event == "PLAYER_LOGIN") then
            -- unregister event
            _self:UnregisterEvent("PLAYER_LOGIN")

            -- show loaded message
            self:GetService("chat"):Write("AddonLoaded");

            -- broadcast version
            self:GetService("message"):SendToGuild(self:GetModel("version-broadcast-message"):Create());
        end
    end);
end

-- create addon
_G.dkpLootMaster = DkpLootMasterAddon:Create();