--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define service
AuctionService = {};
AuctionService.__index = AuctionService;

--- Initialize service.
function AuctionService:Initialize()
    -- prepare auctions
    self.auctions = {};
    self.auctionViews = {};
end

--- Start new auction.
-- @param loot Loot to start auction for.
-- @param parentView Parent view.
-- @param isTest Indicates if is test.
function AuctionService:StartAuction(loot, parentView, isTest)
    -- set loot in auction
    loot.isInAuction = true;

    -- check participants
    if (loot.participants == nil) then
        -- auction next loot
        addon:GetService("queue"):AuctionNextLoot();
        return;
    end

    -- get services
    local playerService = addon:GetService("player");
    local stocksService = addon:GetService("stocks");
    local groupService = addon:GetService("group");

    -- get item
    local item = Item:CreateFromItemID(loot.itemId);

    -- wait until loaded
    item:ContinueOnItemLoad(function()
         -- get model
        local BidSystem = addon:GetModel("bid-system");

        -- prepare auction
        local auction = nil;

        -- check system
        if (Settings.system == BidSystem.GoldOnly) then
            auction = addon:GetModel("auction"):Create(item, Settings.gold.minimum, Settings.gold.step,
                          Settings.gold.mustCarry, 0, 0, 0, Settings.bidTime, isTest, Settings.greedMode);
        elseif (Settings.system == BidSystem.DkpOnly) then
            auction = addon:GetModel("auction"):Create(item, 0, 0, false, Settings.dkp.step,
                          Settings.dkp.bonusRaidMaximum, Settings.dkp.threshold, Settings.bidTime, 
                          isTest, Settings.greedMode);
        else
            auction = addon:GetModel("auction"):Create(item, Settings.gold.minimum, Settings.gold.step,
                          Settings.gold.mustCarry, Settings.dkp.step, Settings.dkp.bonusRaidMaximum,
                          Settings.dkp.threshold, Settings.bidTime, isTest, Settings.greedMode);
        end

        -- add auction participants
        for i, lootParticipant in ipairs(loot.participants) do
            -- check if is online
            if (not groupService:IsOffline(lootParticipant.name)) then
                table.insert(auction.participants, {
                    name = lootParticipant.name,
                    stock = stocksService:GetDkp(lootParticipant.name),
                    canBid = lootParticipant.canBid,
                    passed = (not lootParticipant.canBid)
                });
            end
        end

        -- add auction to started auctions
        self.auctions[auction.id] = auction;

        -- create and show auction view
        local auctionView = addon:CreateView("auction");
        self.auctionViews[auction.id] = auctionView;
        auctionView:Show(auction, true);

        -- set position
        if (parentView ~= nil) then
            auctionView.view:ClearAllPoints();
            auctionView.view:SetPoint("TOPLEFT", parentView.view, "TOPRIGHT", 0, 0);
        end

        -- send auction start
        local messageService = addon:GetService("message");
        local newAuctionMessage = addon:GetModel("new-auction-message");
        for i, participant in ipairs(auction.participants) do
            messageService:SendToPlayer(participant.name,
                newAuctionMessage:Create(auction, participant.canBid, participant.stock));
        end

        -- check quick end
        if (self:CheckQuickEnd(auction)) then
            return;
        end

        -- start timer with inial 5 seconds plus 1 delay second
        addon:GetService("timer"):Start("auction:" .. auction.id, auction.bidTime + 5 + 1, function(seconds)
            if (seconds == 0) then
                self:EndAuction(auction);
            end
        end);
    end);
end

--- End auction.
-- @param auction Auction to end.
function AuctionService:EndAuction(auction)
    -- stop timer
    addon:GetService("timer"):Stop("auction:" .. auction.id);

    -- get last bid
    local lastBid = auction:GetLastBid();

    -- find winner
    auction.winner, auction.rolls = self:FindWinner(auction, lastBid);

    -- remove auction
    self.auctions[auction.id] = nil;

    -- hide auction view
    if (self.auctionViews[auction.id] ~= nil) then
        self.auctionViews[auction.id]:Hide();
        self.auctionViews[auction.id] = nil;
    end

    -- send to all players
    addon:GetService("message"):SendToPlayers(auction:GetParticipantNames(),
        addon:GetModel("auction-ended-message"):Create(auction, lastBid));

    -- check if is test
    if (auction.isTest) then
        return;
    end

    -- reduce dkp
    addon:GetService("stocks"):ReduceDkp(auction.winner, lastBid.dkp);

    -- store auction
    addon:GetService("history"):StoreAuction(auction);

    -- auction next loot
    addon:GetService("queue"):AuctionNextLoot();
end

--- Find auction winner.
-- @param auction Auction to find winner for.
-- @param lastBid Last bid of auction.
function AuctionService:FindWinner(auction, lastBid)
    -- get players of last bid
    local players = lastBid.players;
    local rolls = {};

    -- get message service
    local messageService = addon:GetService("message");

    -- find winner as long as there are more than 1 player
    while (#players > 1) do
        -- prepare players
        local highestPlayers = {};
        local highestRoll = 0;

        -- roll all players
        for i, player in ipairs(players) do
            -- roll
            local roll = random(1, 100);

            -- add roll
            table.insert(rolls, {
                player = player,
                roll = roll
            });

            -- send roll
            messageService:SendToPlayers(auction:GetParticipantNames(),
                addon:GetModel("player-rolled-message"):Create(auction.id, player, roll));

            -- check roll
            if (roll > highestRoll) then
                highestRoll = roll;
                highestPlayers = {player};
            elseif (roll == highestRoll) then
                table.insert(highestPlayers, player);
            end
        end

        -- refresh players
        players = highestPlayers;
    end

    -- check if has one player
    if (#players > 0) then
        return players[1], rolls;
    end
    return nil, rolls;
end

--- Check if message is auction message.
-- @param prefix Message prefix.
-- @param sender Sender of message.
-- @param message received message. 
function AuctionService:CheckMessage(prefix, sender, message)
    -- check if is bid
    local bidMessage = addon:GetModel("bid-message");
    if (prefix == bidMessage.prefix) then
        -- check bid
        self:CheckBid(sender, bidMessage:Parse(message));
        return;
    end

    -- check if is pass
    local passMessage = addon:GetModel("pass-message");
    if (prefix == passMessage.prefix) then
        -- check pass
        self:CheckPass(sender, passMessage:Parse(message));
        return;
    end
end

--- Check bid of player.
-- @param player Name of player who placed the bid.
-- @paran bid Placed bid information.
function AuctionService:CheckBid(player, bid)
    -- get and check auction 
    local auction = self.auctions[bid.auctionId];
    if (auction == nil) then
        return;
    end

    -- get services
    local GreedMode = addon:GetModel("greed-mode");
    local messageService = addon:GetService("message");
    local timerService = addon:GetService("timer");

    -- get last bid
    local lastBid = auction:GetLastBid();

    -- get gold and dkp of last bid
    local lastBidGold = (auction.greedMode == GreedMode.GoldAndDkp and bid.isGreed == false and lastBid.isGreed == true) and 0 or lastBid.gold;
    local lastBidDkp = (bid.isGreed == false and lastBid.isGreed == true and ((auction.greedMode ~= GreedMode.Disabled and bid.dkp > 0) or auction.greedMode == GreedMode.GoldAndDkp)) and 0 or lastBid.dkp;

    -- check if gold bid
    if (bid.gold > 0) then
        -- check if dkp bid
        if ((bid.isGreed == false and auction.greedMode == GreedMode.MainSpecDkpOnly) or 
            lastBidDkp > 0 or 
            lastBidGold + auction.goldStep > bid.gold or 
            auction.goldMinimum > bid.gold) then
            -- something went wron, resend to player
            messageService:SendToPlayer(player, addon:GetModel("bid-placed-message"):Create(auction.id, lastBid));
            return;
        end

        -- add gold bid
        lastBid = auction:AddBid(player, bid.gold, 0, bid.isGreed);
    elseif (bid.dkp > 0) then
        -- check if dkp to low
        if (lastBidDkp < auction.dkpThreshold and lastBidDkp + auction.dkpStep > bid.dkp or lastBidDkp >=
            auction.dkpThreshold and lastBidDkp > bid.dkp) then
            -- something went wron, resend to player
            messageService:SendToPlayer(player, addon:GetModel("bid-placed-message"):Create(auction.id, lastBid));
            return;
        end

        -- check if same dkp bid
        if (lastBidDkp == bid.dkp) then
            -- check if palyer already added
            for i, bidPlayer in ipairs(lastBid.players) do
                if (bidPlayer == player) then
                    return;
                end
            end

            -- add player
            table.insert(lastBid.players, player);
        else
            -- add dkp bid
            lastBid = auction:AddBid(player, 0, bid.dkp, bid.isGreed);
        end
    else
        return;
    end

    -- send bit placed
    messageService:SendToPlayers(auction:GetParticipantNames(),
        addon:GetModel("bid-placed-message"):Create(auction.id, lastBid));

    -- refresh auction view
    if (self.auctionViews[auction.id] ~= nil) then
        self.auctionViews[auction.id]:Refresh();
    end

    -- check quick end
    if (self:CheckQuickEnd(auction)) then
        return;
    end

    -- restart timer plus 1 delay second
    timerService:Start("auction:" .. auction.id, auction.bidTime + 1, function(seconds)
        if (seconds == 0) then
            self:EndAuction(auction);
        end
    end);
end

--- Check pass of player.
-- @param player Name of player who passed.
-- @paran pass Auction pass information.
function AuctionService:CheckPass(player, pass)
    -- get and check auction 
    local auction = self.auctions[pass.auctionId];
    if (auction == nil) then
        return;
    end

    -- set participant passend
    for i, participant in ipairs(auction.participants) do
        -- check if player is participant
        if (participant.name == player) then
            participant.passed = true;
            self:CheckQuickEnd(auction);
            return;
        end
    end
end

--- Check if auction can be ended now.
-- @param auction Auction to check.
-- @return True, if auction was ended.
function AuctionService:CheckQuickEnd(auction)
    -- get last bid
    local lastBid = auction:GetLastBid();

    -- get open players
    for i, participant in ipairs(auction.participants) do
        -- check if passed or participant uses greed and main bid
        if (not participant.passed) then
            -- check if is higest bidder
            local highesBidder = false;
            for i, player in ipairs(lastBid.players) do
                if (player == participant.name) then
                    highesBidder = true;
                    break
                end
            end

            -- check if is greed
            if (lastBid.isGreed == true and not highesBidder) then
                return false;
            end

            -- check gold bid
            if (lastBid.dkp == 0 and not highesBidder) then
                return false;
            end

            -- check if below threshold and can increase
            if (lastBid.dkp < auction.dkpThreshold and participant.stock >= lastBid.dkp + auction.dkpStep and not highesBidder) then
                return false;
            end

            -- check if can increase
            if (lastBid.dkp >= auction.dkpThreshold and highesBidder and participant.stock > lastBid.dkp and #lastBid.players > 1) then
                return false;
            end

            -- check if bonus dk rached and can increase
            if (lastBid.dkp >= auction.dkpThreshold and participant.stock >= lastBid.dkp and not highesBidder) then
                return false;
            end
        end
    end

    -- end auction
    self:EndAuction(auction);
    return true;
end

-- register service
addon:RegisterService(AuctionService, "auction");
