--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define service
BidService = {};
BidService.__index = BidService;

--- Initialize service.
function BidService:Initialize()
    self.participations = {};
end

-- check if message is auction message
function BidService:CheckMessage(prefix, sender, message)
    -- check if bid placed
    local bidPlacedMessage = addon:GetModel("bid-placed-message");
    if (prefix == bidPlacedMessage.prefix) then
        -- display dkp bid
        self:DisplayBid(sender, bidPlacedMessage:Parse(message));
        return;
    end

    -- check if player rolled
    local playerRolledMessage = addon:GetModel("player-rolled-message");
    if (prefix == playerRolledMessage.prefix) then
        -- display roll
        self:DisplayRoll(sender, playerRolledMessage:Parse(message));
        return;
    end

    -- check if is auction start
    local newAuctionMessage = addon:GetModel("new-auction-message");
    if (prefix == newAuctionMessage.prefix) then
        -- show new auction
        self:ShowNewAuction(sender, newAuctionMessage:Parse(message));
        return;
    end

    -- check if auction ended
    local auctionEndedMessage = addon:GetModel("auction-ended-message");
    if (prefix == auctionEndedMessage.prefix) then
        self:AuctionEnded(sender, auctionEndedMessage:Parse(message));
        return;
    end
end

-- check if message is auction message
function BidService:ShowNewAuction(auctioneer, message)
    -- get item
    local item = Item:CreateFromItemID(message.itemId);

    -- wait until loaded
    item:ContinueOnItemLoad(function()
        -- create auction
        local auction = addon:GetModel("auction"):Create(item, message.goldMinimum, message.goldStep,
                            message.mustCarryGold, message.dkpStep, message.dkpBonusRaidMaximum, message.dkpThreshold,
                            message.bidTime, message.isTest, message.greedMode);
        auction.id = message.auctionId;

        -- create participation
        local participation = addon:GetModel("participation"):Create(auction, auctioneer, message.dkpStock);
        self.participations[auction.id] = participation;

        -- check if can bid
        if (not message.canBid) then
            -- show cannot bid message
            addon:GetService("chat"):Write("BidCannotBid", auction.item.link);
            return;
        end

        -- check if not loot master and player has item already
        if (not addon:GetService("player"):IsCurrentPlayer(auctioneer) and GetItemCount(message.itemId, true) > 0) then
            -- show item in inventory message and pass
            addon:GetService("chat"):Write("BidItemInInventory", auction.item.link);
            self:Pass(participation);
            return;
        end

        -- check options
        self:CheckOptions(participation);

        -- check auto pass
        if (self:CheckAutoPass(participation)) then
            -- show item in inventory message and pass
            addon:GetService("chat"):Write("BidAutoPass", auction.item.link);
            self:Pass(participation);
            return;
        end

        -- create view
        participation.view = addon:CreateView("bid");

        -- show view
        participation.view:Show(participation);

        -- play sound
        PlaySound(5274, "Master");
    end)
end

-- check options of bidder
function BidService:CheckOptions(participation)
    -- get values
    local GreedMode = addon:GetModel("greed-mode");
    local auction = participation.auction;
    local lastBid = auction:GetLastBid();

    -- get player service
    local playerService = addon:GetService("player");

    -- evaluate if current play is highest bidder
    participation.highestBidder = false;
    for i, player in ipairs(lastBid.players) do
        if (playerService:IsCurrentPlayer(player)) then
            participation.highestBidder = true;
            break
        end
    end

    -- reset options
    local options = participation.options;
    options.canPass = true;
    options.minGold = 0;
    options.maxGold = 0;
    options.minDkp = 0;
    options.maxDkp = 0;
    options.quickDkp = 0;

    -- cehck if main greed must be selected
    if (auction.greedMode > 0 and participation.isGreed == nil) then
        options.selectMainGreed = true;
    else
        options.selectMainGreed = false;
    end

    -- get dkp and gold of last bid
    local lastBidDkp = (auction.greedMode ~= GreedMode.Disabled and participation.isGreed == false and lastBid.isGreed == true) and 0 or lastBid.dkp;
    local lastBidGold = (auction.greedMode == GreedMode.GoldAndDkp and participation.isGreed == false and lastBid.isGreed == true) and 0 or lastBid.gold;

    -- evaluate if bonus dkp reached
    local stock = participation.stock;
    local bonusDkpReached = lastBidDkp >= auction.dkpThreshold;
    local canCatchUp = lastBidDkp <= stock;
    local canIncrease = lastBidDkp < stock;

    -- check if is highest bidder
    if (participation.highestBidder) then
        -- player can not pass
        options.canPass = false;

        -- check if multipla players are higest bidder 
        if (bonusDkpReached and canIncrease and #lastBid.players > 1) then
            options.quickDkp = lastBidDkp + 1;
        end
        return;
    end

    -- check if gold is generally allowed to bid by greed mode
    local goldForbiddenByGreedMode = (auction.greedMode == GreedMode.GoldAndDkp and participation.isGreed == true and not lastBid.isGreed) or
        (auction.greedMode == GreedMode.MainSpecDkpOnly and participation.isGreed == false);

    -- check if no dkp bid before and gold allowed
    if (not goldForbiddenByGreedMode and lastBidDkp <= 0 and auction.goldStep > 0) then
        -- calculate next minimum gold bid
        local minimumGoldBid = math.max(auction.goldMinimum, lastBidGold + auction.goldStep);
        
        -- check if player has enough gold
        local goldStock = auction.mustCarryGold and (GetMoney() / 10000) or (1000000);
        if (minimumGoldBid <= goldStock) then
            options.minGold = minimumGoldBid;
            options.maxGold = goldStock;
        end
    end

    -- check if dkp allowed
    if (auction.dkpStep <= 0 or (auction.greedMode ~= GreedMode.Disabled and participation.isGreed == true and not lastBid.isGreed)) then
        return;
    end

    -- evaluate next default skp bid
    local nextDefaultDkpBid = lastBidDkp + auction.dkpStep;

    -- check if catch up should be shown
    if (bonusDkpReached and canIncrease and canCatchUp) then
        -- catch up with low dkp bid
        options.minDkp = lastBidDkp;
        options.maxDkp = lastBidDkp;
        -- check if can increase
    elseif (nextDefaultDkpBid < auction.dkpThreshold and nextDefaultDkpBid < stock) then
        -- increase by dkp step with low dkp bid
        options.minDkp = nextDefaultDkpBid;
        options.maxDkp = math.min(auction.dkpThreshold - auction.dkpStep, stock);
    end

    -- check if high dkp bid possible
    if (nextDefaultDkpBid < stock and stock < auction.dkpThreshold) then
        -- use stock as high dkp bid
        options.quickDkp = stock;
        -- check if next low step is threshold
    elseif (nextDefaultDkpBid <= auction.dkpThreshold and stock >= auction.dkpThreshold) then
        -- next hight dkp bid is threshold
        options.quickDkp = auction.dkpThreshold;
        -- check if same can be bid and is stocks
    elseif (bonusDkpReached and stock == lastBidDkp) then
        -- next hight dkp bid is current plus one
        options.quickDkp = lastBidDkp;
        -- check if same can be bid and is stocks
    elseif ((not bonusDkpReached) and stock == nextDefaultDkpBid) then
        -- next hight dkp bid is next step
        options.quickDkp = nextDefaultDkpBid;
        -- check if more can be bid over threshold
    elseif (bonusDkpReached and canIncrease) then
        -- next hight dkp bid is current plus one
        options.quickDkp = lastBidDkp + 1;
    end
end

--- Check if has no options.
function BidService:CheckAutoPass(participation)
    -- check if highest bidder
    if (participation.highestBidder or participation.passed) then
        return false;
    end

    -- get last bid
    local lastBid = participation.auction:GetLastBid();

    -- check if no dkp bid and no bid possible (auctioneer waiting for possible gold bid)
    local options = participation.options;
    return (lastBid.dkp <= 0 and options.minGold <= 0 and options.minDkp <= 0 and options.quickDkp <= 0) or
        (lastBid.isGreed == false and participation.isGreed == true);
end

--- Set greed.
function BidService:SetGreed(participation, isGreed)
    -- set is greed flag
    participation.isGreed = isGreed;

    -- check options
    self:CheckOptions(participation);

    -- check auto pass
    if (self:CheckAutoPass(participation)) then
        -- pass
        self:Pass(participation);
    elseif (participation.view ~= nil) then
        -- update bid buttons
        participation.view:UpdateBidButtons(true);
    end
end

--- Place bid.
function BidService:Bid(participation, gold, dkp)

    -- local slotID = GetInventorySlotInfo(slotName)
    -- local itemLink = GetInventoryItemLink("player", slotID)
    -- local _, _, _, ilvl = GetItemInfo(itemLink)
    -- where slotName is just each one of these with "Slot" concatenated at the end of the String:
    -- Code:
    -- local slots = {"Head", "Neck", "Shoulder", "Back", "Chest", "Shirt", "Tabard", "Wrist", "Waist", "Legs", "Feet", "Hands", "Finger0", "Finger1", "Trinket0", "Trinket1", "MainHand", "SecondaryHand"}

    -- send bid
    addon:GetService("message"):SendToPlayer(participation.auctioneer,
        addon:GetModel("bid-message"):Create(participation.auction.id, gold, dkp, participation.isGreed ~= false));
end

--- Pass.
function BidService:Pass(participation)
    -- send bid
    addon:GetService("message"):SendToPlayer(participation.auctioneer,
        addon:GetModel("pass-message"):Create(participation.auction.id));

    -- set passed
    participation.passed = true;
    if (participation.view ~= nil) then
        participation.view:Update(participation, false, true);
    end
end

--- Close auction window.
function BidService:Close(participation)
    -- set passed
    participation.view:Hide();
    participation.view = nil;
end

-- auction ended
function BidService:AuctionEnded(auctioneer, message)
    -- get and check participation
    local participation = self.participations[message.auctionId];
    if (participation == nil or participation.auctioneer ~= auctioneer) then
        return;
    end

    -- get services
    local chatService = addon:GetService("chat");
    local localeService = addon:GetService("locale");
    local playerService = addon:GetService("player");
    local historyService = addon:GetService("history");
    local GreedMode = addon:GetModel("greed-mode"); 

    -- get auction and set winner
    local auction = participation.auction;
    auction.winner = message.winner;

    -- close view
    if (participation.view ~= nil) then
        participation.view:Hide();
        participation.view = nil;
    end

    -- check player
    if (auction.winner == nil) then
        -- write no winner
        chatService:Write("NoBidsFor", auction.item.link);
    else
        -- get icons
        local skullTarget = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8:10|t";
        local xTarget = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_7:10|t";

        -- write winner
        local winner = playerService:GetShortName(auction.winner);
        if (message.dkp > 0) then
            -- get greed info
            local greedInfo = auction.greedMode ~= nil and auction.greedMode ~= GreedMode.Disabled 
                and (message.isGreed 
                    and (" - " .. xTarget .. " Second Spec") 
                    or (" - " .. skullTarget .. " Main Spec")) 
                or "";

            -- write text
            chatService:Write("PlayerWinsItem", winner, auction.item.link, message.dkp, "DKP", greedInfo);
        else
             -- get greed info
            local greedInfo = auction.greedMode == GreedMode.GoldAndDkp 
                and (message.isGreed 
                    and (" - " .. xTarget .. " Second Spec") 
                    or (" - " .. skullTarget .. " Main Spec")) 
                or "";

            -- write text
            chatService:Write("PlayerWinsItem", winner, auction.item.link, message.gold, "Gold", greedInfo);
        end

        -- check if you won
        if (playerService:IsCurrentPlayer(winner)) then
            -- play sound
            PlaySoundFile("sound/character/gnome/gnomevocalmale/gnomemalecongratulations01.ogg", "Master");
        end
    end

    -- store auction 
    if (not auction.isTest and participation.auctioneer ~= playerService.current) then
        historyService:StoreAuction(participation.auction);
    end

    -- remove participation
    self.participations[message.auctionId] = nil;
end

-- display dkp bid
function BidService:DisplayBid(auctioneer, message)
    -- get and check participation
    local participation = self.participations[message.auctionId];
    if (participation == nil or participation.auctioneer ~= auctioneer) then
        return;
    end

    -- get last bid
    local auction = participation.auction;
    local lastBid = auction:GetLastBid();

    -- store old values
    local highestBidderBefore = participation.highestBidder;
    local highestPlayerAmountBefore = #lastBid.players;

    -- get gold and dkp of last bid
    local GreedMode = addon:GetModel("greed-mode");
    local lastBidGold = (auction.greedMode == GreedMode.GoldAndDkp and message.bid.isGreed == false and lastBid.isGreed == true) and 0 or lastBid.gold;
    local lastBidDkp = (message.bid.isGreed == false and lastBid.isGreed == true and ((auction.greedMode ~= GreedMode.Disabled and message.bid.dkp > 0) or auction.greedMode == GreedMode.GoldAndDkp)) and 0 or lastBid.dkp;

    -- add bid
    if (lastBidGold == message.bid.gold and lastBidDkp == message.bid.dkp) then
        -- update players of bid
        lastBid.players = message.bid.players;
    else
        -- add new bid
        table.insert(auction.bids, message.bid);
        lastBid = message.bid;
    end

    -- check view
    if (participation.view == nil) then
        return;
    end

    -- check options
    self:CheckOptions(participation);

    -- check auto pass
    if (self:CheckAutoPass(participation)) then
        -- pass
        self:Pass(participation);
    end
    
    -- update view
    participation.view:Update(participation, false, false);

    -- check if wsa highest bidder change or player changed
    if ((highestBidderBefore and not participation.highestBidder) or
        (highestBidderBefore and highestPlayerAmountBefore < #lastBid.players)) then
        -- play sound
        PlaySound(5274, "Master");
    end
end

--- Display the given roll.
function BidService:DisplayRoll(auctioneer, roll)
    -- get and check participation
    local participation = self.participations[roll.auctionId];
    if (participation == nil or participation.auctioneer ~= auctioneer) then
        return;
    end

    -- add roll to auction
    table.insert(participation.auction.rolls, {
        player = roll.player,
        roll = roll.roll
    });

    -- get short name
    local player = addon:GetService("player"):GetShortName(roll.player);

    -- write message
    addon:GetService("chat"):Write("PlayerRolled", player, roll.roll);
end

-- register service
addon:RegisterService(BidService, "bid");
