--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define service
GroupService = {};
GroupService.__index = GroupService;

--- Initialize service.
function GroupService:Initialize()
    -- reset values
    self.players = {};
    if (VersionChecks == nil) then VersionChecks = {} end;
end

--- Players updated.
function GroupService:PlayersUpdated()
    -- check if queue disabled
    if (not addon:GetService("queue").watching) then
        return;
    end

    -- check all players
    self:CheckAllPlayers();
end

--- Check all players.
function GroupService:CheckAllPlayers()
    -- get current players
    local currentPlayers = self:GetCurrentPlayers();

    -- store players
    self.players = currentPlayers;

    -- check players
    for i, player in ipairs(currentPlayers) do
        -- check player
        self:CheckPlayer(player.name, false);
    end 

    -- wait for checks
    self:WaitForChecks();
end

--- Wait for checks.
function GroupService:WaitForChecks()
    -- validate checkes
    addon:GetService("timer"):Wait("PlayerCheck", 5, function()
        -- validate check
        self:ValidateCheck();
    end);
end

--- Get all current players in group.
-- @return List of all players currently in group.
function GroupService:GetCurrentPlayers()
    -- prepare current players
    local currentPlayers = {};

    -- check if is in raid
    if (IsInRaid()) then
        for i = 1, GetNumGroupMembers() do
            table.insert(currentPlayers, self:GetPlayer("raid" .. i));
        end
    -- check if is group
    elseif (IsInGroup()) then
        -- add current player
        table.insert(currentPlayers, self:GetPlayer("player"));

        -- check party members
        for i = 1, 4 do
            local player = self:GetPlayer("party" .. i);
            if (player ~= nil) then
                table.insert(currentPlayers, player);
            end
        end
    else
        -- add current player
        table.insert(currentPlayers, self:GetPlayer("player"));
    end
    return currentPlayers;
end

--- Get all current players names in group.
-- @return List of all player names currently in group.
function GroupService:GetCurrentPlayerNames()
    -- prepare current players
    local currentPlayerNames = {};
    local playerService = addon:GetService("player");

    -- check if is in raid
    if (IsInRaid()) then
        for i = 1, GetNumGroupMembers() do
            -- get and check player name
            local playerName = playerService:CheckName(GetUnitName("raid" .. i));
            if (playerName ~= nil) then
                table.insert(currentPlayerNames, playerName);
            end
        end
    -- check if is group
    elseif (IsInGroup()) then
        -- get and check player name
        local playerName = playerService:CheckName(GetUnitName("player"));
        if (playerName ~= nil) then
            table.insert(currentPlayerNames, playerName);
        end

        -- check party members
        for i = 1, 4 do
            -- get and check player name
            playerName = playerService:CheckName(GetUnitName("party" .. i));
            if (playerName ~= nil) then
                table.insert(currentPlayerNames, playerName);
            end
        end
    else
        -- get and check player name
        local playerName = playerService:CheckName(GetUnitName("player"));
        if (playerName ~= nil) then
            table.insert(currentPlayerNames, playerName);
        end
    end
    return currentPlayerNames;
end

--- Get player information by selector.
-- @param selector Selector of player.
-- @return Player name, class and online status.
function GroupService:GetPlayer(selector)
    -- get and check player name
    local playerName = addon:GetService("player"):CheckName(GetUnitName(selector));
    if (playerName == nil) then
        return nil;
    end

    -- get player class
    local _, playerClass = UnitClass(selector);

    -- get player online
    local playerIsOnline = UnitIsConnected(selector);

    -- get player version
    local versionCheck = VersionChecks[playerName];
    local version = versionCheck and versionCheck.version or nil;
    local versionTimeout = versionCheck and versionCheck.timeout or false;

    -- create player
    return {
        name = playerName,
        class = playerClass,
        isOnline = playerIsOnline,
        version = version,
        versionTimeout = versionTimeout
    };
end

--- Check player.
-- @param playerName Name of player to check.
-- @param forceCheck Indicates if check should be forced again.
-- @return True, if player check was sent.
function GroupService:CheckPlayer(playerName, forceCheck)
    -- check if is offline
    if (self:IsOffline(playerName)) then
        -- reset player check if timeout before
        if (VersionChecks[playerName] ~= nil and VersionChecks[playerName].timeout) then
            VersionChecks[playerName] = nil;
        end
        return;
    end

    -- check if player check already sent
    if (VersionChecks[playerName] ~= nil and (not forceCheck)) then
        addon:LogTrace("GroupService", "CheckPlayer", "Player %s already checked. Skipping player.", playerName);
        return;
    end

    -- reset check
    VersionChecks[playerName] = nil;

    -- get service and model
    local messageService = addon:GetService("message");
    local stocksService = addon:GetService("stocks");
    local versionCheckMessage = addon:GetModel("version-check-message");

    -- send check to player
    if (messageService:SendToPlayer(playerName, versionCheckMessage:Create(addon.version))) then
        -- log player check sent
        addon:LogDebug("GroupService", "CheckPlayer", "Player-Check was sent to %s.", playerName);

        -- add check
        VersionChecks[playerName] = {
            version = nil,
            timeout = false,
            notified = false
        };
        return;
    end

    -- log player check not sent
    addon:LogDebug("GroupService", "CheckPlayer", "Player-Check was NOT sent to %s.", playerName);
end

--- Check if player is offline.
-- @param playerName Name of player to check if online.
function GroupService:IsOffline(playerName)
    -- check all players
    for i, player in ipairs(self.players) do
        if (player.name == playerName) then
            return (not player.isOnline);
        end
    end
    return false;
end

--- Validate all player checks.
function GroupService:ValidateCheck()
    -- get service
    local stocksService = addon:GetService("stocks");

    -- validate all player checks
    for playerName, check in pairs(VersionChecks) do 
        if (check.version == nil) then
            -- set timeout
            check.timeout = true;

            -- send version timeout
            stocksService:VersionTimeout(playerName);

            -- check if watching
            if (addon:GetService("queue").watching and not check.notified) then
                -- notify player addon required
                self:NotifyPlayerAddonRequired(playerName, check);
            end
        end
    end
end

--- Notify addon required. 
function GroupService:NotifyAllAddonRequired()
    -- iterate players
    for i, playerName in ipairs(self:GetCurrentPlayerNames()) do
        -- check if player has time out
        local check = VersionChecks[playerName];
        if (check ~= nil and check.timeout and not check.notified) then
            self:NotifyPlayerAddonRequired(playerName, check);
        end
    end
end

--- Notify addon required. 
function GroupService:NotifyPlayerAddonRequired(playerName, check)
    -- send whisper to play to install the addon
    SendChatMessage(addon:GetService("locale"):Get("AddonRequired"), "WHISPER", "Common", playerName);
    addon:LogDebug("GroupService", "NotifyPlayerAddonRequired", "Player %s whispered to uprgade version.", playerName);
    check.notified = true;
end

--- Check message.
function GroupService:CheckMessage(prefix, sender, message)
    -- check if queue accepted
    local playerVersionMessage = addon:GetModel("player-version-message");
    if (prefix == playerVersionMessage.prefix) then
        -- player sent version
        self:PlayerSentVersion(sender, playerVersionMessage:Parse(message));
        return;
    end
end

--- Player sent version.
function GroupService:PlayerSentVersion(playerName, message)
    -- get player check
    local check = VersionChecks[playerName];

    -- check if palyer checked
    if (check ~= nil) then
        -- store version
        check.version = message.version;

        -- send version received
        addon:GetService("stocks"):VersionReceived(playerName, message.version);
    end
end

--- Get participants of item.
-- @param itemLink Link of item to get participants of.
-- @param itemClassID Class id of item to get participants of.
-- @param itemSubClassID Sub class id of item to get participants of.
-- @return List of all current players with "canBid" = true if allowed to bid.
function GroupService:GetParticipants(itemLink, itemClassID, itemSubClassID)
    -- get and check allowed player classes
    local allowedPlayerClasses = addon:GetService("item"):GetAllowedPlayerClasses(itemLink, itemClassID, itemSubClassID);
    if (#allowedPlayerClasses == 0) then
        return {};
    end

    -- get ignored classes
    local playerClasses = {};
    for i, class in ipairs(allowedPlayerClasses) do
        playerClasses[class] = true;
    end

    -- prepare participants
    local participants = {};

    -- check all players
    for i, player in ipairs(self.players) do
        table.insert(participants, {
            name = player.name,
            canBid = playerClasses[player.class] ~= nil
        })
    end
    return participants;
end

-- register service
addon:RegisterService(GroupService, "group");