--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define service
ItemService = {};
ItemService.__index = ItemService;

--- Initialize service.
function ItemService:Initialize()
    -- get ignore item classes and special items
    self.ignoreItemClasses = addon:GetModel("ignore-item-classes"):Create();
    self.specialItems = addon:GetModel("special-items"):Create();

    -- get item player classes and define all classes
    self.itemPlayerClasses = addon:GetModel("item-player-classes"):Create();
    self.allClasses = {"WARRIOR", "PALADIN", "DRUID", "ROGUE", "PRIEST", "MAGE", "WARLOCK", "HUNTER", "SHAMAN"};

    -- create tooltip
    self.tooltip = CreateFrame("GameTooltip", "DLM_ItemService_Tooltip", nil, "GameTooltipTemplate");
    self.tooltip:UnregisterAllEvents();

    -- load classes
    self.classes = {};
    FillLocalizedClassList(self.classes);
end

--- Check if is ignored item.
-- @param itemId Id of item to check if sould be ignored.
-- @param itemClassID Class id of item to check if sould be ignored.
-- @param itemSubClassID Sub class id of item to check if sould be ignored.
-- @return True, if should be ignored.
function ItemService:IsIgnoredItem(itemId, itemClassID, itemSubClassID)
    -- check if is special item
    if (self.specialItems[itemId] ~= nil) then
        return (not self.specialItems[itemId]);
    end

    -- check if item class is ignored
    local itemClass = self.ignoreItemClasses[itemClassID];
    if (itemClass ~= nil) then
        -- check if ignored
        if (itemClass.ignore) then
            return true;
        end

        -- check if sub class is ignored
        local itemSubClass = itemClass[itemSubClassID];
        if (itemSubClass ~= nil and itemSubClass.ignore) then
            return true;
        end
    end
    return false;
end

--- Get allowed player classes of item.
-- @param itemLink Link of item to get allowed player classes of.
-- @param itemClassID Class id of item to get allowed player classes of.
-- @param itemSubClassID Sub class id of item to get allowed player classes of.
-- @return List of all allowed player classes.
function ItemService:GetAllowedPlayerClasses(itemLink, itemClassID, itemSubClassID)
    -- get an check tooltip classes
    local tooltipClasses = self:GetTooltipPlayerClasses(itemLink);
    if (#tooltipClasses > 0) then
        return tooltipClasses;
    end

    -- check if item class is known
    local itemClass = self.itemPlayerClasses[itemClassID];
    if (itemClass == nil) then
        -- class not specified, allow all player classes
        return self.allClasses;
    end

    -- check if sub class is known
    local itemSubClass = itemClass[itemSubClassID];
    if (itemSubClass == nil) then
        -- sub class not specified, allow all player classes
        return self.allClasses;
    end

    -- return player classes
    return itemSubClass;
end

--- Get player classes from tool tip.
-- @param itemLink Link of item to get player classes of tooltip.
-- @return List of all player classes in tooltip.
function ItemService:GetTooltipPlayerClasses(itemLink)
    -- show tooltip
    self.tooltip:SetOwner(UIParent, "ANCHOR_NONE");
    self.tooltip:SetHyperlink(itemLink);

    -- prepare result
    local result = {};

    -- get delimiter and search pattern
    local delimiter = ", ";
    local itemClassesAllowedPattern = _G.ITEM_CLASSES_ALLOWED:gsub("%%s", "%(%.%+%)");

    -- iterate all tooltip lines
    for i = 1, self.tooltip:NumLines() or 0 do
        -- get line
        local line = getglobal(self.tooltip:GetName() .. "TextLeft" .. i);
        if line and line.GetText then
            -- get line text
            local text = line:GetText() or "";

            -- find and check classes 
            local classesText = text:match(itemClassesAllowedPattern);
            if (classesText) then
                -- classes found, hide tool tip
                self.tooltip:Hide();

                -- fix delimiter
                if (LIST_DELIMITER and LIST_DELIMITER ~= "" and classesText:find(LIST_DELIMITER:gsub("%%s", ""))) then
                    delimiter = LIST_DELIMITER:gsub("%%s", "");
                elseif (PLAYER_LIST_DELIMITER and PLAYER_LIST_DELIMITER ~= "" and
                    classesText:find(PLAYER_LIST_DELIMITER)) then
                    delimiter = PLAYER_LIST_DELIMITER;
                end

                -- read classes
                for className in string.gmatch(classesText .. delimiter, "(.-)" .. delimiter) do
                    for class, translation in pairs(self.classes) do
                        if (translation == className) then
                            -- add class 
                            table.insert(result, class);
                        end
                    end
                end
            end
        end
    end

    -- hide tooltip
    self.tooltip:Hide();
    return result;
end

-- register service
addon:RegisterService(ItemService, "item");
