--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define service
QueueService = {};
QueueService.__index = QueueService;

--- Initialize service.
function QueueService:Initialize()
    -- init database
    if (Queue == nil) then
        Queue = {}
    end
    if (Queue.watched == nil) then
        Queue.watched = false
    end
    if (Queue.startedWatchInGroup == nil) then
        Queue.startedWatchInGroup = false
    end
    if (Queue.loot == nil) then
        Queue.loot = {}
    end
    if (Queue.lootIgnore == nil) then
        Queue.lootIgnore = {}
    end
    if (Kills == nil) then
        Kills = {};
    end

    -- reset view
    self.view = nil;
    self.watching = false;
    self.auctioning = false;
    self.askedLootMaster = false;
end

--- Check if loot should be watched.
function QueueService:CheckLootWatch()
    -- check if is loot master
    local isLootMaster = addon:GetService("player"):CurrentIsLootMaster();
    local isInGroup = GetNumGroupMembers() > 0;

    -- check if already watching
    if (self.watching) then
        -- check if is not loot master anymore
        if (not isLootMaster or (not isInGroup and Queue.startedWatchInGroup)) then
            self:StopLootWatch();
        end
        return;
    end

    -- check if watch should be resteted (no question)
    if (Queue.watched and
        ((isInGroup and Queue.startedWatchInGroup and isLootMaster) or
            ((not isInGroup) and (not Queue.startedWatchInGroup)))) then
        self:StartLootWatch(true);
        return;
    end

    -- check if is loot master
    if (not isLootMaster or self.askedLootMaster) then
        return;
    end

    -- ask user to confirm
    local localeService = addon:GetService("locale");
    addon:CreateView("confirmation"):Show(
        localeService:Get("QueueLootMaster"), 
        localeService:Get("QueueAskStart"),
        function()
            -- watch loot
            self:StartLootWatch(false);
        end
    );
        
    -- set asked loot master
    self.askedLootMaster = true;
end

--- Start watching of loot.
function QueueService:StartLootWatch(startNow)
    -- get setvices
    local chatService = addon:GetService("chat");
    local localeService = addon:GetService("locale");

    -- get group members
    local groupMembers = GetNumGroupMembers();

    -- check if is loot master and not allone
    if (groupMembers > 0 and not addon:GetService("player"):CurrentIsLootMaster()) then
        chatService:Write("NotAllowedToStartQueue");
        return;
    end

    -- check if already watching
    if (self.watching) then
        chatService:Write("QueueAlreadyRunning");
        return;
    end

    -- check if should be started
    if (not startNow) then
        -- reset version checks
        VersionChecks = {};

        -- show stocks with "can start"
        addon:GetService("stocks"):ShowStocks(true);
        return;
    end

    -- check if kills msut be deleted
    if (not Queue.watched) then
        -- reset kills
        Kills = {};
    end

    -- set watching
    self.watching = true;
    Queue.watched = true;
    Queue.startedWatchInGroup = groupMembers > 0;

    -- remove in auction flag
    for i, loot in ipairs(Queue.loot) do
        loot.isInAuction = false;
    end

    -- creae view and show view
    if (self.view == nil) then
        self.view = addon:CreateView("queue");
    end
    self.view:Show();

    -- check group
    addon:GetService("group"):PlayersUpdated();

    -- write started
    chatService:Write("QueueStarted");
end

--- Stop watching loot.
function QueueService:StopLootWatch()
    -- reset values
    self.watching = false;
    self.askedLootMaster = false;
    Queue.watched = false;
    Queue.startedWatchInGroup = false;
    Queue.lootIgnore = {}

    -- reset version checks
    VersionChecks = {};

    -- hide view
    if (self.view ~= nil) then
        self.view:Hide();
    end

    -- write queue stopped
    addon:GetService("chat"):Write("QueueStopped");
end

--- Boss was killed.
-- @param bossName Name of killed boss.
function QueueService:BossKilled(bossName)
    -- check boxx name
    if (bossName == nil or not self.watching) then
        return;
    end

    -- add kill
    Kills[bossName] = addon:GetService("group"):GetCurrentPlayerNames();
end

--- Check current loot.
function QueueService:CheckCurrentLoot()
    -- check if watching
    if (not self.watching) then
        addon:LogTrace("QueueService", "CheckCurrentLoot", "Service is not watching.")
        return;
    end

    -- prepare loot added
    local lootAdded = false;

    -- iterate loot items
    for slot = 1, GetNumLootItems() do
        -- check if has item
        if (LootSlotHasItem(slot)) then
            -- add loot
            lootAdded = self:AddLootBySlot(slot) or lootAdded;
        end
    end

    -- check if loot added
    if (lootAdded and self.view ~= nil) then
        -- refresh view
        self.view:Show();
    end
end

-- add loot by slot
function QueueService:AddLootBySlot(slot)
    -- get createure id
    local creatureId = GetLootSourceInfo(slot);

    -- check if ignored
    if (Queue.lootIgnore[creatureId .. ":" .. slot] ~= nil) then
        return;
    end

    -- check if loot alredy added
    for i, loot in ipairs(Queue.loot) do
        -- check if creature and slot already added
        if (loot.creatureId == creatureId and loot.slot == slot) then
            -- loot alredy added, do not add again
            return false;
        end
    end

    -- load values
    local itemIcon, itemName, quantity, currencyId, itemQuality, isLocked, isQuestItem = GetLootSlotInfo(slot);

    -- check if is relevant
    if (currencyId or isQuestItem or itemQuality < 1 or itemQuality == 5 or isLocked) then
        addon:LogDebug("QueueService", "AddLoot", "Ignoring loot of slot %d because currency, quest item, poor or legendary or locked.", slot);
        return false;
    end

    -- get and check link
    local itemLink = GetLootSlotLink(slot);
    if (itemLink == nil) then
        addon:LogDebug("QueueService", "AddLoot", "Item link can not be loaded of slot %d.", slot);
        return false;
    end

    -- load more item data
    local itemId, _, _, _, _, itemClassID, itemSubClassID = GetItemInfoInstant(itemLink);

    -- check if should be ignored
    if (addon:GetService("item"):IsIgnoredItem(itemId, itemClassID, itemSubClassID)) then
        return false;
    end

    -- get and check participants
    local participants = addon:GetService("group"):GetParticipants(itemLink, itemClassID, itemSubClassID);
    if (#participants == 0) then
        return false;
    end

    -- add entry per quantity
    local lootModel = addon:GetModel("loot");
    for i = 1, quantity do
        table.insert(Queue.loot, lootModel:Create(creatureId, slot, itemId, itemLink, itemIcon, participants));
    end

    -- log loot added
    addon:LogDebug("QueueService", "AddLoot", "Added loot by slot. creatureId: %s, slot: %d, itemId: %s, quantity: %d",
        creatureId, slot, itemId, quantity);
    return true;
end

-- add loot by slot
function QueueService:AddLootByItemLink(itemLink)
    -- load more item data
    local itemId, _, _, _, itemIcon, itemClassID, itemSubClassID = GetItemInfoInstant(itemLink);

    -- get and check participants
    local participants = addon:GetService("group"):GetParticipants(itemLink, itemClassID, itemSubClassID);
    if (#participants == 0) then
        addon:GetService("chat"):Write("QueueInvalidItem");
        return false;
    end

    -- add entry per quantity
    local lootModel = addon:GetModel("loot");
    table.insert(Queue.loot, lootModel:Create("Manual", -1, itemId, itemLink, itemIcon, participants));

    -- log loot added
    addon:LogDebug("QueueService", "AddLoot", "Added loot by item link. itemId: %s", itemId);
    return true;
end

-- remove loot
function QueueService:RemoveLoot(lootId)
    -- check loot
    for i, loot in ipairs(Queue.loot) do
        -- check if iod matches
        if (loot.id == lootId) then
            -- remove llot
            table.remove(Queue.loot, i);

            -- check if can be ignored
            if (loot.creatureId ~= "Manual") then
                Queue.lootIgnore[loot.creatureId .. ":" .. loot.slot] = true;
            end

            -- log removed
            addon:LogTrace("QueueService", "RemoveLoot", "Loot removed. creatureId: %s, slot: %d", loot.creatureId,
                loot.slot);
            return;
        end
    end
end

--- Start auctioning.
function QueueService:StartAuctioning()
    -- check if alrey auctioning
    if (self.auctioning) then
        return;
    end

    -- start auctioning
    self.auctioning = true;

    -- auction next loot
    self:AuctionNextLoot();
end

--- Auctio next loot.
function QueueService:AuctionNextLoot()
    -- check if not auctioning
    if (not self.auctioning) then
        return;
    end

    -- iterate creatures
    for i, loot in ipairs(Queue.loot) do
        -- check if is in auction
        if (loot.isInAuction) then
            -- remove loot
            self:RemoveLoot(loot.id);
            break
        end
    end

    -- start auction
    for i, loot in ipairs(Queue.loot) do
        addon:GetService("auction"):StartAuction(loot, self.view, false);
        self.view:Show();
        return;
    end

    -- no loot found, stop auctioning
    self.auctioning = false;
    self.view:Show();
end

-- check if has queue
function QueueService:HasQeueue()
    return #Queue.loot > 0;
end

-- register service
addon:RegisterService(QueueService, "queue");
