--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define service
StocksService = {};
StocksService.__index = StocksService;

--- Initialize service.
function StocksService:Initialize()
    -- init database
    if (Stocks == nil) then
        Stocks = {}
    end
    self.stocksView = nil;
    self.raidEndView = nil;
end

--- Show stocks.
-- @param startQueue Indicates if loot master can be stated.
function StocksService:ShowStocks(startQueue)
    -- check if view must be created
    if (self.stocksView == nil) then
        self.stocksView = addon:CreateView("stocks");
    end

    -- show view
    self.stocksView:Show(startQueue);

    -- check if can start queue
    if (startQueue) then
        -- check all players
        addon:GetService("group"):CheckAllPlayers();
    end
end

--- Show raid end.
function StocksService:ShowRaidEnd()
    -- -- check if view must be created
    -- if (self.raidEndView == nil) then
    --     self.raidEndView = addon:CreateView("raid-end");
    -- end

    -- -- show raid end
    -- self.raidEndView:Show();
end

--- Version received by player.
function StocksService:VersionReceived(playerName, version)
    -- check if view created
    if (self.stocksView == nil) then
        return;
    end

    -- update version
    self.stocksView:UpdateVersion(playerName, version);
end

--- Version received not by player.
function StocksService:VersionTimeout(playerName)
    -- check if view created
    if (self.stocksView == nil) then
        return;
    end

    -- update version
    self.stocksView:VersionTimeout(playerName);
end

--- Show import.
function StocksService:ShowImport(onImport, onClose)
    -- get services and create view
    local localeService = addon:GetService("locale");
    local playerService = addon:GetService("player");
    local importView = addon:CreateView("copy");

    -- prepare lines
    local lines = {};

    -- get stocks
    for i, stock in ipairs(Stocks) do
        -- get player names
        local playerNames = {};
        for j, player in ipairs(stock.players) do
            table.insert(playerNames, playerService:GetShortName(player));
        end

        -- add line
        table.insert(lines, table.concat(playerNames, ",") .. ":" .. (Settings.dkp.threshold + stock.bonusDkp))
    end

    -- show import ivew
    importView:Show("DLMSettingsStocks", localeService:Get("ImportDkpData"),
        localeService:Get("ImportDkpDataDescription"), table.concat(lines, "\n"), true, function(value)
            -- import
            self:Import(value);

            -- trigger callbacks
            onImport();
        end, onClose);
end

--- Import stocks.
function StocksService:Import(text)
    -- prepare all player names and get message service 
    local allPlayerNames = {};
    local messageService = addon:GetService("message");

    -- clear stocks
    Stocks = {};

    -- get realm name
    local realmName = GetRealmName();

    -- iterate lines
    for line in string.gmatch(text, "([^\n]+)") do
        -- prepare stock
        local stock = {
            isNew = false
        };

        -- get line values
        for lineValue in string.gmatch(line, "([^:]+)") do
            -- check if players set
            if (stock.players == nil) then
                -- prepare players
                stock.players = {};

                -- check player names
                for playerName in string.gmatch(lineValue, "([^,]+)") do
                    if (playerName ~= nil) then
                        -- trim player name
                        playerName = messageService:TrimString(playerName);
                        if (string.len(playerName) > 0) then
                            -- check if already imported
                            if (allPlayerNames[string.lower(playerName)] == nil) then
                                -- insert player
                                table.insert(stock.players, addon:GetService("player"):CheckName(playerName));
                                allPlayerNames[string.lower(playerName)] = true;
                            end
                        end
                    end
                end
            else
                -- read dkp
                local dkp = tonumber(lineValue);
                stock.dkp = math.min(dkp, Settings.dkp.threshold + Settings.dkp.bonusRaidMaximum);

                -- check if maximum saving activated
                if (Settings.dkp.bonusSavingMaximum > 0) then
                    stock.bonusDkp =
                        math.min(math.max(dkp - Settings.dkp.threshold, 0), Settings.dkp.bonusSavingMaximum);
                else
                    stock.bonusDkp = math.max(dkp - Settings.dkp.threshold, 0);
                end
                break
            end
        end

        -- check stock
        if (stock.players ~= nil and #stock.players > 0 and stock.dkp ~= nil and stock.bonusDkp ~= nil) then
            -- add to stocks
            table.insert(Stocks, stock);
        end
    end

    -- show stocks
    self:ShowStocks();
end

-- reduce player dkp
function StocksService:ReduceDkp(player, dkp)
    -- check player and dkp
    if (player == nil or dkp == nil or dkp == 0) then
        return;
    end

    -- calculate bonus dkp to reduce
    local bonusDkp = math.max(dkp - Settings.dkp.threshold, 0);

    -- get stock
    local stock = self:GetStock(player);

    -- reduce dkp
    if (bonusDkp > 0) then
        stock.dkp = 0;
    else
        stock.dkp = math.max(math.min(stock.dkp, Settings.dkp.threshold) - dkp, 0);
    end

    -- reduce bonus dkp
    stock.bonusDkp = math.max(stock.bonusDkp - bonusDkp, 0);
    stock.isNew = false;
end

--- Remove the given stock.
-- @param stock Stock to remove.
function StocksService:RemoveStock(stock)
    for i, s in ipairs(Stocks) do
        -- check if is current player
        if (s.players[1] == stock.players[1]) then
            table.remove(Stocks, i);
            return;
        end
    end
end

--- Reset dkp.
-- @param stock Stock to reset dkp for. If null, all stocks will be reseted.
function StocksService:ResetDkp(stock)
    -- check if one stock should be reseted
    if (stock ~= nil) then
        -- reset single stock
        stock.dkp = Settings.dkp.threshold + math.min(Settings.dkp.bonusRaidMaximum, stock.bonusDkp);
        return;
    end

    -- reset all stocks
    for i, s in pairs(Stocks) do
        s.dkp = Settings.dkp.threshold + math.min(Settings.dkp.bonusRaidMaximum, s.bonusDkp);
    end
end

--- Get stock of player.
-- @param player Name of player to get stocks for.
function StocksService:GetStock(player)
    -- iterate stocks
    for i, stock in pairs(Stocks) do
        -- iterate players
        for j, stockPlayer in pairs(stock.players) do
            if (player == stockPlayer) then
                return stock;
            end
        end
    end

    -- create new stock
    local stock = {
        players = {player},
        dkp = Settings.dkp.threshold,
        bonusDkp = 0,
        isNew = true
    };
    table.insert(Stocks, stock);
    return stock;
end

--- Get DKP of player.
-- @param player Name of player to get DKP for.
function StocksService:GetDkp(player)
    -- check settings
    if (not addon:GetService("settings"):DkpEnabled()) then
        return 0;
    end

    -- get stock
    local stock = self:GetStock(player);

    -- return dkp
    return stock.dkp, stock.bonusDkp;
end

--- Assign main character to stock.
-- @param stock Stock with single characters to assign to main character.
-- @param callback Callback function which is triggered after successful assignment.
-- @param parent Parent view.
function StocksService:AssignMainCharacter(stock, callback, parent)
    -- check player amount
    if (#stock.players > 1) then
        return;
    end

    -- show view
    addon:CreateView("stocks-main-character"):Show(stock, callback, parent);
end

--- Detach character of stock.
-- @param stock Stock with multiple characters to assign to detach one character from.
-- @param selectedName Pre selected name of twink.
-- @param callback Callback function which is triggered after successful detach.
-- @param parent Parent view.
function StocksService:DetachCharacter(stock, selectedName, callback, parent)
    -- check player amount
    if (#stock.players < 2) then
        return;
    end

    -- show view
    addon:CreateView("stocks-detach-character"):Show(stock, selectedName, callback, parent);
end

-- register service
addon:RegisterService(StocksService, "stocks");
