--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define view
AuctionView = {};
AuctionView.__index = AuctionView;

--- Show view.
-- @param auction auction to show.
-- @param canManage Indicates if auction can be managed.
function AuctionView:Show(auction, canManage)
    -- get services
    local uiService = addon:GetService("ui");
    local localeService = addon:GetService("locale");

    -- check if view already created
    if (self.view == nil) then
        -- create view
        local view = uiService:CreateView("DLMAuction", 320, 200, localeService:Get("Auction"));
        self.view = view;

        -- add close button
        local closeButton = CreateFrame("Button", nil, view, "UIPanelCloseButton");
        closeButton:SetHeight(24);
        closeButton:SetWidth(24);
        closeButton:SetPoint("TOPRIGHT", -7, -7);
        closeButton:SetScript("OnClick", function()
            view:Hide();
        end);
        self.closeButton = closeButton;

        -- create scroll frame 
        local scrollFrame, scrollChild = uiService:CreateScrollFrame(view);
        scrollFrame:SetPoint("TOPLEFT", 10, -36);
        scrollFrame:SetPoint("BOTTOMRIGHT", -10, 10);
        scrollChild:SetWidth(scrollFrame:GetWidth());
        self.scrollChild = scrollChild;

        -- reset row
        self.rows = {};
    end

    -- store values
    self.auction = auction;
    self.canManage = canManage;

    -- check if can manage
    if (canManage) then
        self.closeButton:Hide();
    else
        self.closeButton:Show();
    end

    -- refresh view
    self:Refresh();

    -- show view
    self.view:Show();
end

-- Refresh view.
function AuctionView:Refresh()
    -- hide rows
    for i in pairs(self.rows) do
        self.rows[i]:Hide();
    end

    -- clear rows
    local entries = {};

    -- get services
    local localeService = addon:GetService("locale");
    local playerService = addon:GetService("player");
    local GreedMode = addon:GetModel("greed-mode"); 

    -- add start
    table.insert(entries, {
        icon = self.auction.item.icon,
        title = localeService:Get("AuctionStarted"),
        content = self.auction.item.link
    });

    -- add bids
    for i, bid in ipairs(self.auction.bids) do
        -- get players
        local playerNameList = {}
        for i, player in ipairs(bid.players) do
            if (playerService:IsCurrentPlayer(player)) then
                table.insert(playerNameList, "|cff00ff00" .. playerService:GetShortName(player));
            else
                table.insert(playerNameList, "|cffffffff" .. playerService:GetShortName(player));
            end
        end

        -- get icons
        local skullTarget = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8:10|t";
        local xTarget = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_7:10|t";

        if (bid.dkp > 0) then
            -- get greed info
            local greedInfo = self.auction.greedMode ~= nil and self.auction.greedMode ~= GreedMode.Disabled 
                and (bid.isGreed 
                    and (" - " .. xTarget .. " Second Spec") 
                    or (" - " .. skullTarget .. " Main Spec")) 
                or "";

            table.insert(entries, 1, {
                icon = "Interface\\Icons\\Inv_misc_head_dragon_bronze",
                title = string.format("%s: %d |cffDA8CFF%s|cffffffff%s", localeService:Get("BidPlaced"), bid.dkp, localeService:Get("Dkp"), greedInfo),
                content = table.concat(playerNameList, ", ");
            });
        else
            -- get greed info
            local greedInfo = self.auction.greedMode == GreedMode.GoldAndDkp 
                and (bid.isGreed 
                    and (" - " .. xTarget .. " Second Spec") 
                    or (" - " .. skullTarget .. " Main Spec")) 
                or "";

            table.insert(entries, 1, {
                icon = "Interface\\Icons\\INV_Misc_Coin_01",
                title = string.format("%s: %d |cffffd800%s|cffffffff%s", localeService:Get("BidPlaced"), bid.gold, localeService:Get("Gold"), greedInfo),
                content = table.concat(playerNameList, ", ");
            });
        end
    end

    -- add rolls
    for i, roll in ipairs(self.auction.rolls) do
        table.insert(entries, 1, {
            icon = "Interface\\Icons\\spell_shadow_auraofdarkness",
            content = localeService:Get("PlayerRolled", playerService:GetShortName(roll.player), roll.roll)
        });
    end

    -- add winner
    if (self.auction.winner ~= nil) then
        table.insert(entries, 1, {
            icon = "Interface\\Icons\\spell_nature_thunderclap",
            content = localeService:Get("PlayerWins", playerService:GetShortName(self.auction.winner))
        });
    elseif(not self.canManage) then
        table.insert(entries, 1, {
            icon = "Interface\\Icons\\inv_enchant_shardnexuslarge",
            content = localeService:Get("NoBids")
        });
    end

    -- add entries
    local top = 0;
    for i, entry in ipairs(entries) do
        -- create frame
        local row = CreateFrame("Button", nil, self.scrollChild, BackdropTemplateMixin and "BackdropTemplate");
        local height = entry.title ~= nill and 34 or 20;
        row:SetPoint("TOPLEFT", 0, -top);
        row:SetPoint("BOTTOMRIGHT", self.scrollChild, "TOPRIGHT", -28, -(top + height));
        row:SetBackdrop({
            bgFile = [[Interface\Buttons\WHITE8x8]]
        });

        -- add frame to frames^^
        table.insert(self.rows, row);

        -- check odd index
        if (i - math.floor(i / 2) * 2 == 0) then
            row:SetBackdropColor(0.1, 0.1, 0.1, 0.8);
        else
            row:SetBackdropColor(0.15, 0.15, 0.15, 0.8);
        end

        -- add icon
        local icon = row:CreateTexture(nil, "ARTWORK");
        icon:SetPoint("TOPLEFT", 6, -(height / 2 - 8));
        icon:SetWidth(16);
        icon:SetHeight(16);
        icon:SetTexture(entry.icon);

        -- add title text
        if (entry.title ~= nil) then
            local titleText = row:CreateFontString(nil, "OVERLAY", "GameFontNormal");
            titleText:SetPoint("TOPLEFT", 30, -8);
            titleText:SetPoint("BOTTOMRIGHT", -6, 22);
            titleText:SetJustifyH("LEFT");
            titleText:SetTextColor(1, 1, 1);
            titleText:SetText(entry.title);
        end

        -- add content text
        local contentText = row:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        contentText:SetPoint("TOPLEFT", row, "BOTTOMLEFT", 30, 21);
        contentText:SetPoint("BOTTOMRIGHT", -6, 0);
        contentText:SetJustifyH("LEFT");
        contentText:SetTextColor(1, 1, 1);
        contentText:SetText(entry.content);
        top = top + height;
    end

    -- set scroll height
    self.scrollChild:SetHeight(top);
end

--- Hide view.
function AuctionView:Hide()
    -- check if view created
    if (self.view ~= nil) then
        self.view:Hide();
        self.view = nil;
    end
end

-- register view
addon:RegisterView(AuctionView, "auction");
