--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define view
BidView = {};
BidView.__index = BidView;

--- Show view.
function BidView:Show(participation)
    -- prepare values
    self.timeout = 5;

    -- create title
    local localeService = addon:GetService("locale");
    local title = localeService:Get("BidNow");
    if (participation.stock > 0) then
        title = title .. " " .. localeService:Get("BidMaxDkp", participation.stock);
    end

    -- create view
    self.view = addon:GetService("ui"):CreateView("DLMBid", 452, 170, title);

    -- crate close button
    self.closeButton = CreateFrame("Button", nil, self.view, "UIPanelCloseButton");
    self.closeButton:SetPoint("TOPRIGHT", -5, -5);
    self.closeButton:SetScript("OnClick", function()
        addon:GetService("bid"):Close(participation);
    end);
    self.closeButton:Hide();

    -- create item info
    self:CreateItemInfo(self.view, participation.auction.item);

    -- create current bid info
    self:CreateCurrentBidInfo(self.view);

    -- create bid buttons
    self:CreateBidButtons(self.view);

    -- update view
    self:Update(participation, true, false);
end

-- hide view
function BidView:Hide()
    self.view:Hide();
end

-- update view
function BidView:Update(participation, initial, passedNow)
    -- store participation
    self.participation = participation;

    -- update current bid info
    self:UpdateCurrentBidInfo();

    -- update buttons
    self:UpdateBidButtons(initial);

    -- check if not passed now
    if (not passedNow) then
        self:RestartTimer(initial);
    end
end

-- create item info
function BidView:CreateItemInfo(view, item)
    -- create item frame
    local itemInfo = CreateFrame("Button", nil, view);
    itemInfo:SetPoint("TOPLEFT", 14, -40);
    itemInfo:SetWidth(420);
    itemInfo:SetHeight(30);

    -- add item icon
    local icon = itemInfo:CreateTexture();
    icon:SetPoint("TOPLEFT", 0, 0);
    icon:SetWidth(30);
    icon:SetHeight(30);
    icon:SetTexture(item.icon);

    -- add item text
    local text = itemInfo:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    text:SetPoint("TOPLEFT", 36, -9);
    text:SetText(item.link);

    -- bind mouse events
    itemInfo:SetScript("OnEnter", function()
        GameTooltip:SetOwner(itemInfo, "ANCHOR_TOPLEFT");
        GameTooltip:SetHyperlink(item.link);
        GameTooltip:Show();
    end);
    itemInfo:SetScript("OnLeave", function()
        GameTooltip:Hide();
    end);
end

-- create current bid info
function BidView:CreateCurrentBidInfo(view)
    -- create bid frame
    local bidInfo = CreateFrame("Button", nil, view);
    bidInfo:SetPoint("TOPLEFT", 14, -81);
    bidInfo:SetWidth(420);
    bidInfo:SetHeight(42);
    bidInfo.bid = nil;

    -- add label
    self.currentBidText = bidInfo:CreateFontString(nil, "OVERLAY", "GameFontNormalLeft");
    self.currentBidText:SetPoint("TOPLEFT", 0, 0);
    self.currentBidText:SetText(addon:GetService("locale"):Get("CurrentBid"));

    -- add text
    self.currentBidPlayerText = bidInfo:CreateFontString(nil, "OVERLAY", "GameFontNormalLeft");
    self.currentBidPlayerText:SetPoint("TOPLEFT", 0, -20);
end

-- update current bid info
function BidView:UpdateCurrentBidInfo()
    -- get last bid and locale service
    local auction = self.participation.auction;
    local GreedMode = addon:GetModel("greed-mode"); 
    local lastBid = auction:GetLastBid();
    local localeService = addon:GetService("locale");

    -- check if last bid is empty
    if (lastBid.gold == 0 and lastBid.dkp == 0 or table.getn(lastBid.players) == 0) then
        self.currentBidText:SetText(localeService:Get("CurrentBid"));
        self.currentBidPlayerText:SetText("|cff777777" .. localeService:Get("NoCurrentBid"));
        return;
    end

    -- prepare bid parts
    local bidParts = {};

    -- get icons
    local skullTarget = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8:10|t";
    local xTarget = "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_7:10|t";

    -- add icon
    local greedInfo = "";
    if (lastBid.dkp > 0) then
        -- get greed info
        greedInfo = auction.greedMode ~= GreedMode.Disabled 
            and (lastBid.isGreed 
                and (" - " .. xTarget .. " Second Spec ") 
                or (" - " .. skullTarget .. " Main Spec ")) 
            or "";

        -- add parts
        table.insert(bidParts, "|TInterface\\Icons\\Inv_misc_head_dragon_bronze:16|t|cffffffff");
        table.insert(bidParts, lastBid.dkp);
        table.insert(bidParts, "|cffDA8CFF" .. localeService:Get("Dkp") .. "|cffffffff:");
    else
        -- get greed info
        greedInfo = auction.greedMode == GreedMode.GoldAndDkp 
            and (lastBid.isGreed 
                and (" - " .. xTarget .. " Second Spec ") 
                or (" - " .. skullTarget .. " Main Spec ")) 
            or "";

        -- add parts
        table.insert(bidParts, "|TInterface\\Icons\\INV_Misc_Coin_01:16|t|cffffffff");
        table.insert(bidParts, lastBid.gold);
        table.insert(bidParts, "|cffffd800" .. localeService:Get("Gold") .. "|cffffffff:");
    end

    -- get player names
    local playerService = addon:GetService("player");
    local playerNameList = {}
    for i, player in ipairs(lastBid.players) do
        if (playerService:IsCurrentPlayer(player)) then
            table.insert(playerNameList, "|cff00ff00" .. playerService:GetShortName(player));
        else
            table.insert(playerNameList, "|cffffffff" .. playerService:GetShortName(player));
        end
    end
    table.insert(bidParts, table.concat(playerNameList, ", "));

    -- set text
    self.currentBidText:SetText(localeService:Get("CurrentBid") .. greedInfo);
    self.currentBidPlayerText:SetText(table.concat(bidParts, " "));
end

-- create bid buttons
function BidView:CreateBidButtons(view)
    -- create button frame
    local buttons = CreateFrame("Frame", nil, view);
    buttons:SetPoint("TOPLEFT", view, "BOTTOMLEFT", 0, 46);
    buttons:SetPoint("BOTTOMRIGHT", 0, 12);

    -- create countdown label
    local countdownLabel = buttons:CreateFontString(nil, "OVERLAY", "GameFontNormalLeft");
    countdownLabel:SetPoint("TOPLEFT", 13, -11);
    countdownLabel:SetTextColor(1, 1, 1);
    countdownLabel:Hide();
    self.countdownLabel = countdownLabel;

    -- get services
    local uiService = addon:GetService("ui");
    local localeService = addon:GetService("locale");
    local bidService = addon:GetService("bid");

    -- create default button
    local defaultButton = uiService:CreateButton(buttons, localeService:Get("BidPass"), function()
        bidService:Pass(self.participation);
    end);
    defaultButton:SetPoint("TOPLEFT", 12, 0);
    defaultButton:SetWidth(100);
    defaultButton:SetHeight(34);
    self.defaultButton = defaultButton;

    -- create main button
    local needButton = uiService:CreateButton(buttons, "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_8:10|t " .. localeService:Get("BidNeed"), function()
        addon:GetService("bid"):SetGreed(self.participation, false);
    end);
    needButton:SetPoint("TOPRIGHT", -144, 0);
    needButton:SetWidth(120);
    needButton:SetHeight(34);
    needButton:Hide();
    self.needButton = needButton;

    -- create second button
    local greedButton = uiService:CreateButton(buttons, "|TInterface\\TargetingFrame\\UI-RaidTargetingIcon_7:10|t " .. localeService:Get("BidGreed"), function()
        addon:GetService("bid"):SetGreed(self.participation, true);
    end);
    greedButton:SetPoint("TOPRIGHT", -12, 0);
    greedButton:SetWidth(120);
    greedButton:SetHeight(34);
    greedButton:Hide();
    self.greedButton = greedButton;

    -- create coin buttons
    self.goldButton = self:CreateCoinButton(buttons, 122, "Interface/Icons/Inv_misc_coin_02", function(value)
        bidService:Bid(self.participation, value, 0);
    end);
    self.dkpButton = self:CreateCoinButton(buttons, 232, "Interface/Icons/Inv_misc_head_dragon_bronze", function(value)
        bidService:Bid(self.participation, 0, value);
    end);
    self.quickDkpButton = self:CreateCoinButton(buttons, 342, "Interface/Icons/Inv_misc_head_dragon_01", function(value)
        bidService:Bid(self.participation, 0, value);
    end);
end

-- update bid buttons
function BidView:UpdateBidButtons(initial)
    -- get options
    local options = self.participation.options;

    -- check if can pass
    if (not self.participation.passed and options.canPass) then
        self.defaultButton:Show();
        self.countdownLabel:Hide();
    else
        self.defaultButton:Hide();
        self.needButton:Hide();
        self.greedButton:Hide();
        self.countdownLabel:Show();
    end

    -- check if main greed should be selected
    if (options.selectMainGreed) then
        self.needButton:Show();
        self.greedButton:Show();
        return;
    else
        self.needButton:Hide();
        self.greedButton:Hide();
    end

    -- set values
    local currentValueChanged = self.goldButton.setValues(options.minGold, options.maxGold, "g", self.participation.auction.goldStep, initial);

    -- check if dkp allowed
    if (options.minDkp > 0 or options.quickDkp > 0) then
        currentValueChanged = self.dkpButton.setValues(options.minDkp, options.maxDkp, "DKP", self.participation.auction.dkpStep, initial) or currentValueChanged;
        currentValueChanged = self.quickDkpButton.setValues(options.quickDkp, options.quickDkp, "DKP", 0, initial) or currentValueChanged;
        self.goldButton:SetPoint("TOPLEFT", 122, -3);
    else
        self.goldButton:SetPoint("TOPLEFT", 342, -3);
    end

    -- check if values changed
    if (self.participation.highestBidder or self.participation.passed or (currentValueChanged and not initial)) then
        self.goldButton:Hide();
        self.dkpButton:Hide();
        self.quickDkpButton:Hide();
    else
        if (options.minGold > 0) then
            self.goldButton:Show();
            self.goldButton.blocked = false;
        end
        if (options.minDkp > 0) then
            self.dkpButton:Show();
            self.dkpButton.blocked = false;
        end
        if (options.quickDkp > 0) then
            self.quickDkpButton:Show();
            self.quickDkpButton.blocked = false;
        end
    end

    -- check if passed
    if (self.participation.passed) then
        -- show close button and dont restart countdown
        self.closeButton:Show();
    end
end

--- Restart timer.
function BidView:RestartTimer(initial)
    -- get service and timer id
    local participation = self.participation;
    local auction = participation.auction;
    local options = participation.options;
    local timerService = addon:GetService("timer");
    local timerId = "bid:" .. auction.id;

    -- get bid time
    local bidTime = auction.bidTime + self.timeout;
    if (initial) then
        bidTime = bidTime + 5;
    end

    -- start timer
    timerService:Start(timerId, bidTime, function(second)
        -- check if 1 seconds passed
        if (second == (bidTime - 1) and not participation.passed and not options.selectMainGreed) then
            if (options.minGold > 0) then
                self.goldButton:Show();
                self.goldButton.blocked = false;
            end
            if (options.minDkp > 0) then
                self.dkpButton:Show();
                self.dkpButton.blocked = false;
            end
            if (options.quickDkp > 0) then
                self.quickDkpButton:Show();
                self.quickDkpButton.blocked = false;
            end
        end

        -- hide winodw on timeout
        if (second == 0) then
            timerService:Stop(timerId);
            self.view:Hide();
        end

        -- update texts
        local localeService = addon:GetService("locale");
        second = math.max(second - self.timeout, 0);
        self.defaultButton:SetText(localeService:Get("BidPass") .. " (" .. second .. ")");
        self.countdownLabel:SetText(localeService:Get("AuctionEndsIn") .. second);
    end);
end

-- create coin button
function BidView:CreateCoinButton(parent, left, iconName, onClick)
    -- create icon button
    local coinFrame = CreateFrame("Frame", nil, parent);
    coinFrame:SetPoint("TOPLEFT", left, -3);
    coinFrame:SetWidth(100);
    coinFrame:SetHeight(34);
    coinFrame:Hide();
    coinFrame.currentValue = 0;
    coinFrame.hovered = false;
    coinFrame.blocked = false;

    -- add bid button
    local bidButton = CreateFrame("Button", nil, coinFrame);
    bidButton:SetPoint("TOPLEFT", 0, 0);
    bidButton:SetWidth(30);
    bidButton:SetHeight(30);
    local bidButtonIcon = bidButton:CreateTexture(nil, "BACKGROUND");
    bidButtonIcon:SetPoint("TOPLEFT", 0, 0);
    bidButtonIcon:SetPoint("BOTTOMRIGHT", 0, 0);
    bidButtonIcon:SetTexture(iconName);

    -- add icon glow
    local bidButtonGlow = coinFrame:CreateTexture(nil, "ARTWORK");
    bidButtonGlow:SetPoint("TOPLEFT", -13.5, 13.5);
    bidButtonGlow:SetWidth(57);
    bidButtonGlow:SetHeight(57);

    -- add text background
    local textBackground = coinFrame:CreateTexture(nil, "BACKGROUND");
    textBackground:SetTexture("Interface/Common/Common-CheckBox-Background");
    textBackground:SetPoint("TOPLEFT", 24, -5);
    textBackground:SetWidth(76);
    textBackground:SetHeight(28);

    -- add text
    local textLabel = coinFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalLeft");
    textLabel:SetPoint("CENTER", textBackground, "CENTER", 0, 4);
    textLabel:SetTextColor(1, 1, 1);

    -- add minus button
    local minusButton = CreateFrame("Button", nil, coinFrame, "UIPanelButtonTemplate");
    minusButton:SetPoint("TOPLEFT", 30, -6);
    minusButton:SetText("-");
    minusButton:SetWidth(20);
    minusButton:SetHeight(17);
    minusButton:Hide();

    -- add plus button
    local plusButton = CreateFrame("Button", nil, coinFrame, "UIPanelButtonTemplate");
    plusButton:SetPoint("TOPRIGHT", -1, -6);
    plusButton:SetText("+");
    plusButton:SetWidth(20);
    plusButton:SetHeight(17);
    plusButton:Hide();

    local hoverOnEnter = function()
        -- set hovered and update look
        coinFrame.hovered = true;
        coinFrame.updateLook();
    end;
    local noHoverOnLeave = function()
        -- set not hovered and update look
        coinFrame.hovered = false;
        coinFrame.updateLook();
    end;

    -- bind coin frame mouse events
    coinFrame:SetScript("OnEnter", hoverOnEnter);
    coinFrame:SetScript("OnLeave", noHoverOnLeave);

    -- bind minus button mouse events
    minusButton:SetScript("OnEnter", hoverOnEnter);
    minusButton:SetScript("OnLeave", noHoverOnLeave);
    minusButton:SetScript("OnClick", function()
        -- decrease value and update look
        coinFrame.currentValue = coinFrame.currentValue - coinFrame.step;
        coinFrame.updateLook();
    end);

    -- bind plus button mouse events
    plusButton:SetScript("OnEnter", hoverOnEnter);
    plusButton:SetScript("OnLeave", noHoverOnLeave);
    plusButton:SetScript("OnClick", function()
        -- increase value and update look
        coinFrame.currentValue = coinFrame.currentValue + coinFrame.step;
        coinFrame.updateLook();
    end);

    -- bind bid button mouse events
    bidButton:SetScript("OnEnter", function()
        -- show button glow
        bidButtonGlow:SetTexture("Interface/Buttons/CheckButtonGlow");
    end);
    bidButton:SetScript("OnLeave", function()
        -- hide button glow
        bidButtonGlow:SetTexture(nil);
    end);
    bidButton:SetScript("OnMouseDown", function()
        bidButton:SetPoint("TOPLEFT", 1, -1);
        bidButtonGlow:SetPoint("TOPLEFT", -14.5, 12.5);
    end);
    bidButton:SetScript("OnMouseUp", function()
        bidButton:SetPoint("TOPLEFT", 0, 0);
        bidButtonGlow:SetPoint("TOPLEFT", -13.5, 13.5);
    end);
    bidButton:SetScript("OnClick", function()
        -- check if not blocked
        if (not coinFrame.blocked) then
            onClick(coinFrame.currentValue);
        end
    end);

    -- set values
    coinFrame.setValues = function(min, max, unit, step, initial)
        -- store values
        coinFrame.min = min;
        coinFrame.max = max;
        coinFrame.unit = unit;
        coinFrame.step = step;

        -- set value
        local currentValueChanged = false;
        if (coinFrame.currentValue < min) then
            -- block button if not initial
            if (not initial) then
                coinFrame.blocked = true;
            end

            -- set minimum value
            coinFrame.currentValue = min;
            currentValueChanged = true;
        elseif (coinFrame.currentValue > max) then
            -- block button if not initial
            if (not initial) then
                coinFrame.blocked = true;
            end

            -- set maximum value
            coinFrame.currentValue = max;
            currentValueChanged = true;
        end

        -- update look
        coinFrame.updateLook();
        return currentValueChanged;
    end;

    -- update look
    coinFrame.updateLook = function()
        -- check if hovered
        local buttonVisible = false;
        if (coinFrame.hovered and coinFrame.step > 0) then
            -- set minus visiblity
            if (coinFrame.currentValue - coinFrame.step >= coinFrame.min) then
                minusButton:Show();
                buttonVisible = true;
            else
                minusButton:Hide();
            end

            -- set plus visiblity
            if (coinFrame.currentValue + coinFrame.step <= coinFrame.max) then
                plusButton:Show();
                buttonVisible = true;
            else
                plusButton:Hide();
            end
        else
            -- hide buttons 
            minusButton:Hide();
            plusButton:Hide();
        end

        -- update text
        if (buttonVisible) then
            textLabel:SetText(coinFrame.currentValue);
        else
            textLabel:SetText(coinFrame.currentValue .. " " .. coinFrame.unit);
        end
    end;
    return coinFrame;
end

-- register view
addon:RegisterView(BidView, "bid");
