--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define view
HistoryView = {};
HistoryView.__index = HistoryView;

--- Show history view.
-- @param searchText Text to search for.
function HistoryView:Show(searchText)
    -- get services
    local uiService = addon:GetService("ui");
    local localeService = addon:GetService("locale");

    -- check if view created
    if (self.view == nil) then
        -- prepare visible auctions list
        self.visibleAuctions = {};

        -- get auctions
        self.auctions = History;
        table.sort(self.auctions, function(a, b)
            return a.started > b.started;
        end);

        -- create view
        local view = uiService:CreateView("DLMHistory", 760, 500, localeService:Get("History"));
        self.view = view;

        -- add close button
        local closeButton = CreateFrame("Button", nil, view, "UIPanelCloseButton");
        closeButton:SetHeight(24);
        closeButton:SetWidth(24);
        closeButton:SetPoint("TOPRIGHT", -5, -5);
        closeButton:SetScript("OnClick", function()
            view:Hide();
            if (self.auctionView ~= nil) then
                self.auctionView.view:Hide();
            end
        end);

        -- add header frame
        local headerFrame = CreateFrame("Frame", nil, view, BackdropTemplateMixin and "BackdropTemplate");
        headerFrame:SetBackdrop({
            bgFile = [[Interface\Buttons\WHITE8x8]]
        });
        headerFrame:SetBackdropColor(0, 0, 0, 0.5);
        headerFrame:SetPoint("TOPLEFT", 10, -36);
        headerFrame:SetPoint("BOTTOMRIGHT", view, "TOPRIGHT", -12, -110);

        -- add date selection drop down
        local dateSelectionLabel = headerFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        dateSelectionLabel:SetPoint("TOPLEFT", 16, -15);
        dateSelectionLabel:SetText(localeService:Get("Date"));
        local dateSelection = CreateFrame("Frame", nil, headerFrame, "UIDropDownMenuTemplate");
        dateSelection:SetPoint("TOPLEFT", -4, -32);
        dateSelection:SetWidth(40);
        self.dateSelection = dateSelection;
        UIDropDownMenu_Initialize(dateSelection, function()
            -- create item
            local item = UIDropDownMenu_CreateInfo();
            item.notCheckable = true;
            item.func = function(_self, arg1, arg2)
                if (arg1 == 0) then
                    UIDropDownMenu_SetText(dateSelection, localeService:Get("All"));
                else
                    UIDropDownMenu_SetText(dateSelection, date(localeService:GetBare("DateFormat"), arg1));
                end
                self.filteredDate = arg1;
                self:AddAuctions();
            end;

            -- add all date
            item.text, item.arg1 = localeService:Get("All"), 0;
            UIDropDownMenu_AddButton(item);

            -- add dates
            local prviousDate = "";
            for i, auction in ipairs(self.auctions) do
                local date = date(localeService:GetBare("DateFormat"), auction.started);
                if (date ~= prviousDate) then
                    item.text, item.arg1 = date, auction.started;
                    UIDropDownMenu_AddButton(item);
                    prviousDate = date;
                end
            end
        end);

        -- add item search box
        local itemSearchLabel = headerFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        itemSearchLabel:SetPoint("TOPLEFT", 156, -15);
        itemSearchLabel:SetText(localeService:Get("Search"));
        local itemSearch = CreateFrame("EditBox", nil, headerFrame, "InputBoxTemplate");
        itemSearch:SetPoint("TOPLEFT", 160, -35);
        itemSearch:SetPoint("BOTTOMRIGHT", headerFrame, "TOPRIGHT", -20, -54);
        itemSearch:SetAutoFocus(false);
        self.itemSearch = itemSearch;
        itemSearch:SetScript("OnTextChanged", function()
            self:AddAuctions();
        end);

        -- add date text
        local dateText = view:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        dateText:SetPoint("TOPLEFT", 22, -118);
        dateText:SetText(localeService:Get("Time"));

        -- add item text
        local itemText = view:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        itemText:SetPoint("TOPLEFT", 162, -118);
        itemText:SetText(localeService:Get("Item"));

        -- add winner text
        local winnerText = view:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        winnerText:SetPoint("TOPLEFT", 518, -118);
        winnerText:SetText(localeService:Get("Winner"));

        -- add bid text
        local bidText = view:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        bidText:SetPoint("TOPRIGHT", -42, -118);
        bidText:SetText(localeService:Get("Bid"));

        -- create scroll frame 
        local scrollFrame, scrollChild = uiService:CreateScrollFrame(view);
        scrollFrame:SetPoint("TOPLEFT", 10, -136);
        scrollFrame:SetPoint("BOTTOMRIGHT", -12, 42);
        scrollChild:SetWidth(scrollFrame:GetWidth());
        self.scrollFrame = scrollFrame;
        self.scrollChild = scrollChild;

        -- create export button
        local exportButton = uiService:CreateButton(view, localeService:Get("HistoryExport"), function()
            self:Export();
        end);
        exportButton:SetPoint("BOTTOMRIGHT", -10, 10);
        exportButton:SetWidth(100);
        exportButton:SetHeight(28);
    else 
        -- refresh
        self:AddAuctions();
    end

    -- check if has auctions
    if (#self.auctions > 0) then
        UIDropDownMenu_SetText(self.dateSelection, date(localeService:GetBare("DateFormat"), self.auctions[1].started));
        self.filteredDate = self.auctions[1].started;
    else
        UIDropDownMenu_SetText(self.dateSelection, localeService:Get("All"));
        self.filteredDate = 0;
    end
    
    -- show view
    self.view:Show();

    -- set search text
    self.itemSearch:SetText(searchText or "");
end

-- add auction
function HistoryView:AddAuctions()
    -- remove auction frames
    for i in pairs(self.visibleAuctions) do
        self.visibleAuctions[i].frame:Hide();
        self.visibleAuctions[i] = nil;
    end

    -- get message service, locale service and date format
    local messageService = addon:GetService("message");
    local localeService = addon:GetService("locale");
    local dateFormat = localeService:GetBare("DateFormat");

    -- get search parts
    local searchText = string.lower(messageService:TrimString(self.itemSearch:GetText()));
    local searchParts = messageService:SplitString(string.gsub(searchText, "%-", "%%-"), ",");

    -- add auctions
    for i, auction in ipairs(self.auctions) do
        -- check date
        if (self.filteredDate == 0 or date(dateFormat, self.filteredDate) == date(dateFormat, auction.started)) then
            -- check if has search parts
            if (#searchParts == 0) then
                -- add auction
                self:AddAuction(auction, localeService);
            else
                for i, part in ipairs(searchParts) do
                    -- trim part
                    part = messageService:TrimString(part);
                    if (string.len(part) > 0 and string.find(string.lower(auction.item.name), part) ~= nil or
                        (auction.winner ~= nil and string.find(string.lower(auction.winner), part))) then
                        -- add auction
                        self:AddAuction(auction, localeService);
                        break
                    end
                end
            end
        end
    end

    -- set scroll height
    self.scrollChild:SetHeight(#self.visibleAuctions * 20);
end

-- add auction
function HistoryView:AddAuction(auction, localeService)
    -- get index
    local index = #self.visibleAuctions + 1;

    -- create frame
    local auctionFrame = CreateFrame("Button", nil, self.scrollChild, BackdropTemplateMixin and "BackdropTemplate");
    local top = (index - 1) * 20;
    auctionFrame:SetPoint("TOPLEFT", 0, -top);
    auctionFrame:SetPoint("BOTTOMRIGHT", self.scrollChild, "TOPRIGHT", -28, -(top + 20));
    auctionFrame:SetBackdrop({
        bgFile = [[Interface\Buttons\WHITE8x8]]
    });
    auctionFrame.index = index;

    -- add frame to frames
    table.insert(self.visibleAuctions, {
        frame = auctionFrame,
        auction = auction
    });

    -- check odd index
    if (index - math.floor(index / 2) * 2 == 0) then
        auctionFrame:SetBackdropColor(0.1, 0.1, 0.1, 0.8);
    else
        auctionFrame:SetBackdropColor(0.15, 0.15, 0.15, 0.8);
    end

    -- add date text
    local dateText = auctionFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    dateText:SetPoint("TOPLEFT", 6, -4);
    dateText:SetText(date(localeService:GetBare("TimeFormat"), auction.started));

    -- add item 
    local itemFrame = CreateFrame("Button", nil, auctionFrame);
    itemFrame:SetPoint("TOPLEFT", 146, -3);
    itemFrame:SetHeight(26);
    itemFrame:SetWidth(350);
    local itemText = itemFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    itemText:SetPoint("TOPLEFT", 0, 0);
    itemText:SetText("|T" .. auction.item.icon .. ":16|t " .. auction.item.link);

    -- handle item mouse enter
    itemFrame:SetScript("OnEnter", function()
        -- show item tool tip
        GameTooltip:SetOwner(itemFrame, "ANCHOR_LEFT");
        GameTooltip:SetHyperlink(auction.item.link);
        GameTooltip:Show();

        -- update row background color
        auctionFrame:SetBackdropColor(0.3, 0.3, 0.3, 0.8);
    end);

    -- handle item mouse leave
    itemFrame:SetScript("OnLeave", function()
        -- hite item tool tip
        GameTooltip:Hide();

        -- update row background color
        if (index - math.floor(index / 2) * 2 == 0) then
            auctionFrame:SetBackdropColor(0.1, 0.1, 0.1, 0.8);
        else
            auctionFrame:SetBackdropColor(0.15, 0.15, 0.15, 0.8);
        end
    end);

    -- handle click event
    itemFrame:SetScript("OnClick", function()
        -- check if shift pressed
        if (IsShiftKeyDown()) then
            -- filter by item
            self.itemSearch:SetText(auction.item.name);
        end
    end);

    -- handle mouse down event
    itemFrame:SetScript("OnMouseDown", function(_, button)
        -- check right button
        if (button == "RightButton") then
            self:ShowActions(auction, auctionFrame);
        end
    end);

    -- check winner
    if (auction.winner ~= nil) then
        -- add winner text
        local winnerText = auctionFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        winnerText:SetPoint("TOPLEFT", 500, -4);
        winnerText:SetPoint("BOTTOMRIGHT", auctionFrame, "BOTTOMRIGHT", -100, -4);
        winnerText:SetJustifyH("LEFT");
        winnerText:SetJustifyV("TOP");
        winnerText:SetTextColor(1, 1, 1);
        winnerText:SetText(addon:GetService("player"):GetShortName(auction.winner));
    end

    -- add bid text
    local bidText = auctionFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    bidText:SetPoint("TOPRIGHT", -6, -4);
    local bid = auction.bids[#auction.bids];
    if (bid ~= null) then
        if (bid.dkp > 0) then
            bidText:SetText("|cffffffff" .. bid.dkp .. " |cffDA8CFF" .. localeService:Get("Dkp"));
        else
            bidText:SetText("|cffffffff" .. bid.gold .. " |cffffd800" .. localeService:Get("Gold"));
        end
    else
        bidText:SetTextColor(1, 1, 1);
        bidText:SetText(localeService:Get("NoBids"));
    end

    -- handle row mouse enter
    auctionFrame:SetScript("OnEnter", function()
        -- update background color
        auctionFrame:SetBackdropColor(0.2, 0.2, 0.2);
    end);

    -- handle row mouse leave
    auctionFrame:SetScript("OnLeave", function()
        -- update background color
        if (index - math.floor(index / 2) * 2 == 0) then
            auctionFrame:SetBackdropColor(0.05, 0.05, 0.05);
        else
            auctionFrame:SetBackdropColor(0.1, 0.1, 0.1);
        end
    end);

    -- handle mouse down event
    auctionFrame:SetScript("OnMouseDown", function(_, button)
        -- check right button
        if (button == "RightButton") then
            self:ShowActions(auction, auctionFrame);
        end
    end);
end

--- Show auction.
function HistoryView:ShowAuction(auction)
    -- crete view
    if (self.auctionView == nil) then
        -- create auction view
        self.auctionView = addon:CreateView("auction");
    end

    -- show auction
    self.auctionView:Show(auction, false);
    self.auctionView.view:ClearAllPoints();
    self.auctionView.view:SetPoint("CENTER", self.view, "CENTER");
end

--- Show actions.
function HistoryView:ShowActions(auction, frame)
    -- create conext menu and get locale service
    local contextMenu = CreateFrame("Frame", "DLMHistory_ContextMenu", UIParent, "UIDropDownMenuTemplate");
    local localeService = addon:GetService("locale");

    -- init drop down
    UIDropDownMenu_Initialize(contextMenu, function(_self, level)
        -- add main entries
        UIDropDownMenu_AddButton({
            text = localeService:Get("Auction"),
            isTitle = true,
            notCheckable = true
        });

        -- check if has winner
        if (auction.winner ~= nil) then
            -- add show course
            UIDropDownMenu_AddButton({
                text = localeService:Get("HistoryShowCourse"),
                notCheckable = true,
                func = function()
                    -- show auction
                    self:ShowAuction(auction);
                end
            });
        end

        -- add filter by item
        UIDropDownMenu_AddButton({
            text = localeService:Get("HistoryFilterBy", auction.item.link),
            notCheckable = true,
            func = function()
                -- filter by item
                UIDropDownMenu_SetText(self.dateSelection, localeService:Get("All"));
                self.filteredDate = 0;
                self.itemSearch:SetText(auction.item.name);
            end
        });

        -- check if has winner
        if (auction.winner ~= nil) then
            -- add filter by winner
            local winner = addon:GetService("player"):GetShortName(auction.winner);
            UIDropDownMenu_AddButton({
                text = localeService:Get("HistoryFilterBy", winner),
                notCheckable = true,
                func = function()
                    -- filter by item
                    UIDropDownMenu_SetText(self.dateSelection, localeService:Get("All"));
                    self.filteredDate = 0;
                    self.itemSearch:SetText(winner);
                end
            });
        end

        -- add empty entry
        UIDropDownMenu_AddButton({
            text = nil,
            disabled = true
        });

        -- add remove entry menu
        UIDropDownMenu_AddButton({
            text = localeService:Get("HistoryRemoveEntry"),
            colorCode = "|cffff0000",
            notCheckable = true,
            func = function()
                addon:CreateView("confirmation"):Show(
                    localeService:Get("HistoryRemoveEntry"), 
                    localeService:Get("HistoryRemoveEntryQuestion"),
                    function()
                        addon:GetService("history"):RemoveAuction(auction);
                        self:AddAuctions();
                    end
                );
            end
        });
    end, "MENU");

    -- show drop down
    ToggleDropDownMenu(1, nil, contextMenu, frame, 0, 0);
end

-- export current visible auctions
function HistoryView:Export()
    -- get lines
    local exportLines = {};

    -- get services
    local playerService = addon:GetService("player");
    local localeService = addon:GetService("locale");

    -- iterate visible auctions
    for i, entry in ipairs(self.visibleAuctions) do
        -- check winner
        local auction = entry.auction;
        if (auction.winner ~= nil) then
            -- build line
            local bid = auction.bids[#auction.bids];
            table.insert(exportLines, table.concat({date("%Y-%m-%d", auction.started), date("%H:%M", auction.started),
                                                    playerService:GetShortName(auction.winner), auction.item.name,
                                                    bid.dkp, bid.gold}, ","));
        end
    end

    -- show copy view
    local exportView = addon:CreateView("copy");
    exportView:Show("DLMHistoryExport", localeService:Get("HistoryExport"),
        localeService:Get("HistoryExportDescription"), table.concat(exportLines, "\n"), false, nil, nil);
end

-- register view
addon:RegisterView(HistoryView, "history");
