--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define view
QueueView = {};
QueueView.__index = QueueView;

--- Show view.
function QueueView:Show()
    -- get services
    local uiService = addon:GetService("ui");
    local localeService = addon:GetService("locale");

    -- check if view already created
    if (self.view == nil) then
        -- prepare values
        self.lootFrames = {};
        self.minimized = true;
        self.smallHeight = 82;

        -- create view
        local view = uiService:CreateView("DLMQueue", 270, self.smallHeight, localeService:Get("QueueLootMaster"));
        self.view = view;

        -- add summary text
        local summaryText = view:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall");
        summaryText:SetPoint("TOPLEFT", 16, -30);
        self.summaryText = summaryText;

        -- add minimize button
        local minimizeButton = CreateFrame("Button", nil, view, "UIPanelScrollUpButtonTemplate");
        minimizeButton:SetHeight(18);
        minimizeButton:SetWidth(18);
        minimizeButton:SetPoint("TOPRIGHT", -12, -11);
        minimizeButton:SetScript("OnClick", function()
            self:Minimize()
        end);
        minimizeButton:Hide();
        self.minimizeButton = minimizeButton;

        -- create scroll frame 
        local scrollFrame, scrollChild = uiService:CreateScrollFrame(view);
        scrollFrame:SetPoint("TOPLEFT", 10, -36);
        scrollFrame:SetPoint("BOTTOMRIGHT", -10, 40);
        scrollChild:SetWidth(scrollFrame:GetWidth());
        scrollFrame:Hide();
        self.scrollFrame = scrollFrame;
        self.scrollChild = scrollChild;

        -- create stocks button
        local stocksButton = uiService:CreateIconButton(view, "Inv_misc_head_gnome_02",
                                 localeService:Get("QueueShowPlayerList"), "", function()
                addon:GetService("stocks"):ShowStocks();
            end);
        stocksButton:SetPoint("BOTTOMLEFT", 16, 13);

        -- create history button
        local historyButton = uiService:CreateIconButton(view, "inv_misc_pocketwatch_01",
                                  localeService:Get("StocksShowHistory"), "", function()
                addon:GetService("history"):ShowHistory(nil);
            end);
        historyButton:SetPoint("BOTTOMLEFT", 44, 13);

        -- create end button
        local endButton = uiService:CreateIconButton(view, "Spell_shadow_twilight",
                              localeService:Get("QueueEndLootMaster"), "", function()
                -- ask user to stop
                addon:CreateView("confirmation"):Show(
                    localeService:Get("QueueEndLootMaster"),
                    localeService:Get("QueueEndLootMasterQuestion"), 
                    function()
                        addon:GetService("queue"):StopLootWatch();
                        addon:GetService("stocks"):ShowRaidEnd();
                    end
                );
            end);
        endButton:SetPoint("BOTTOMLEFT", 72, 13);

        -- create loot button
        lootButton = uiService:CreateIconButton(view, "Inv_misc_bag_corefelclothbag",
                         localeService:Get("QueueShowItems"), "", function()
                self:Maximize();
            end, 30);
        lootButton:SetPoint("CENTER", 94, 0);
        lootButton:Hide();
        self.lootButton = lootButton;

        -- create loot amount
        local lootAmountText = lootButton:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        lootAmountText:SetPoint("BOTTOMRIGHT", -3, 2);
        lootAmountText:SetTextColor(1, 1, 1);
        self.lootAmountText = lootAmountText;

        -- create run button
        local runButton = uiService:CreateButton(view, localeService:Get("QueueStartAuctions"), function()
            addon:GetService("queue"):StartAuctioning();
        end);
        runButton:SetPoint("BOTTOMRIGHT", -10, 10);
        runButton:SetWidth(140);
        runButton:SetHeight(28);
        runButton:Hide();
        self.runButton = runButton;

        -- handle drag
        view:SetScript("OnMouseUp", function(_, button)
            self:OnMouseUp(button);
        end);
        view:SetScript("OnReceiveDrag", function(button)
            self:OnMouseUp("LeftButton");
        end);

        -- handle mouse enter
        view:SetScript("OnEnter", function()
            -- get cursor info
            local type, _, itemLink = GetCursorInfo();
            if (type ~= "item" or not itemLink) then
                return;
            end

            view:SetBackdropColor(0.2, 0.2, 0, 0.8);
        end);

        -- handle delete mouse leave
        view:SetScript("OnLeave", function()
            view:SetBackdropColor(0, 0, 0, 0.8);
        end);
    end

    -- check queue amount
    local queueAmount = #Queue.loot;
    if (queueAmount > 0) then
        -- refresh summary
        self.summaryText:SetText(queueAmount > 1 and localeService:Get("QueueItemsOpen", queueAmount) or
                                     localeService:Get("QueueItemOpen"));
        self.summaryText:SetTextColor(1, 1, 1);
        self.lootAmountText:SetText(queueAmount);

        -- check if has queue and is minimize
        if (self.minimized) then
            -- show amximize button
            self.lootButton:Show();
        end
    else
        -- refresh summary
        self.summaryText:SetText(localeService:Get("QueueWatching"));
        self.summaryText:SetTextColor(0.9, 0.9, 0.9);

        -- minimize and hide maximize button
        self:Minimize();
        self.lootButton:Hide();
    end

    -- remove loot frames
    for i in pairs(self.lootFrames) do
        self.lootFrames[i]:Hide();
        self.lootFrames[i] = nil;
    end

    -- iterate creatures
    for i, loot in ipairs(Queue.loot) do
        -- add loot 
        self:AddLoot(loot);
    end

    -- set scroll height
    self.scrollChild:SetHeight(#self.lootFrames * 20);

    -- show view
    self.view:Show();
end

--- On click event.
function QueueView:OnMouseUp(button)
    -- check button
    if (button ~= "LeftButton") then
        return;
    end

    -- get cursor info
    local type, _, itemLink = GetCursorInfo();
    if (type ~= "item" or not itemLink) then
        return;
    end

    -- add item
    if (addon:GetService("queue"):AddLootByItemLink(itemLink)) then
        self:Show();
    end

    -- remove color highlight and clear cursor
    self.view:SetBackdropColor(0, 0, 0, 0.8);
    ClearCursor();
end

--- Minimize view.
function QueueView:Minimize()
    self.view:SetHeight(self.smallHeight);
    self.scrollFrame:Hide();
    self.runButton:Hide();
    self.minimizeButton:Hide();
    self.lootButton:Show();
    self.summaryText:Show();
    self.minimized = true;
end

--- Maximize view.
function QueueView:Maximize()
    self.view:SetHeight(200);
    self.scrollFrame:Show();
    self.runButton:Show();
    self.minimizeButton:Show();
    self.lootButton:Hide();
    self.summaryText:Hide();
    self.minimized = false;
end

-- hide view
function QueueView:Hide()
    -- check view
    if (self.view ~= nil) then
        self.view:Hide()
    end
end

-- add loot
function QueueView:AddLoot(loot)
    -- get index
    local index = #self.lootFrames + 1;

    -- create frame
    local top = (index - 1) * 20;
    local lootFrame = CreateFrame("Button", nil, self.scrollChild, BackdropTemplateMixin and "BackdropTemplate");
    lootFrame:SetPoint("TOPLEFT", 0, -top);
    lootFrame:SetPoint("BOTTOMRIGHT", self.scrollChild, "TOPRIGHT", -28, -(top + 20));
    lootFrame:SetBackdrop({
        bgFile = [[Interface\Buttons\WHITE8x8]]
    });

    -- add frame to frames
    table.insert(self.lootFrames, lootFrame);

    -- check odd index
    if (loot.isInAuction) then
        lootFrame:SetBackdropColor(0, 0.5, 0);
    elseif (index - math.floor(index / 2) * 2 == 0) then
        lootFrame:SetBackdropColor(0.05, 0.05, 0.05);
    else
        lootFrame:SetBackdropColor(0.1, 0.1, 0.1);
    end

    -- add loot icon
    local lootIcon = lootFrame:CreateTexture();
    lootIcon:SetPoint("TOPLEFT", 4, -2);
    lootIcon:SetHeight(16);
    lootIcon:SetWidth(16);
    lootIcon:SetTexture(loot.itemIcon);

    -- add loot text
    local lootText = lootFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    lootText:SetPoint("TOPLEFT", 24, -2);
    lootText:SetPoint("BOTTOMRIGHT", -4, 0);
    lootText:SetJustifyH("LEFT");
    lootText:SetText(loot.itemLink);

    -- create delete button
    local deleteButton = CreateFrame("Button", nil, lootFrame, "UIPanelCloseButton");
    deleteButton:SetHeight(24);
    deleteButton:SetWidth(24);
    deleteButton:SetPoint("TOPRIGHT", -1, 2);
    deleteButton:Hide();

    -- highlight row callback
    local highlightRow = function()
        -- update row background color and show delete button
        lootFrame:SetBackdropColor(0.2, 0.2, 0.2);
        deleteButton:Show();
    end;

    -- revert row callback
    local revertRow = function()
        -- update row background color
        if (index - math.floor(index / 2) * 2 == 0) then
            lootFrame:SetBackdropColor(0.05, 0.05, 0.05);
        else
            lootFrame:SetBackdropColor(0.1, 0.1, 0.1);
        end

        -- hide buttons and tooltip
        deleteButton:Hide();
    end;

    -- handle delete mouse enter
    deleteButton:SetScript("OnEnter", function()
        -- highlight row
        if (not loot.isInAuction) then
            highlightRow();
        end
    end);

    -- handle delete mouse leave
    deleteButton:SetScript("OnLeave", function()
        -- revert row
        if (not loot.isInAuction) then
            revertRow();
        end
    end);

    -- handle delete click 
    deleteButton:SetScript("OnClick", function()
        -- remove loot
        local queueService = addon:GetService("queue");
        queueService:RemoveLoot(loot.id);

        -- refresh view
        self:Show();
    end);

    -- handle loot mouse enter
    lootFrame:SetScript("OnEnter", function()
        -- show loot tool tip
        GameTooltip:SetOwner(lootFrame, "ANCHOR_LEFT");
        GameTooltip:SetHyperlink(loot.itemLink);
        GameTooltip:Show();

        -- highlight row
        if (not loot.isInAuction) then
            highlightRow();
        end
    end);

    -- handle loot mouse leave
    lootFrame:SetScript("OnLeave", function()
        -- hide loot tool tip
        GameTooltip:Hide();

        -- revert row
        if (not loot.isInAuction) then
            revertRow();
        end
    end);
end

-- register view
addon:RegisterView(QueueView, "queue");
