--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define view
SettingsView = {};
SettingsView.__index = SettingsView;

--- Show options view.
-- @param onUpdate Callback, triggered on settings update.
-- @param onClose Callback, triggered on settings view close.
function SettingsView:Show(onUpdate, onClose)
    -- get services
    local uiService = addon:GetService("ui");
    local localeService = addon:GetService("locale");

    -- get models
    local BidSystem = addon:GetModel("bid-system");
    local GreedMode = addon:GetModel("greed-mode");

    -- check if view created
    if (self.view == nil) then
        -- create view
        local view = uiService:CreateView("DLMSettingsStocks", 460, 360, localeService:Get("Settings"));
        self.view = view;

        -- add close button
        local closeButton = CreateFrame("Button", nil, view, "UIPanelCloseButton");
        closeButton:SetHeight(24);
        closeButton:SetWidth(24);
        closeButton:SetPoint("TOPRIGHT", -7, -7);
        closeButton:SetScript("OnClick", function()
            view:Hide()
        end);

        -- add system drop down
        local systemLabel = view:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        systemLabel:SetPoint("TOPLEFT", 16, -40);
        systemLabel:SetText("System");
        local systemSelection = CreateFrame("Frame", nil, view, "UIDropDownMenuTemplate");
        systemSelection:SetPoint("TOPLEFT", -4, -54);
        systemSelection:SetWidth(40);
        self.systemSelection = systemSelection;
        UIDropDownMenu_Initialize(systemSelection, function()
            UIDropDownMenu_AddButton({
                text = localeService:Get("SettingsSystemGoldOnly"),
                tooltipTitle = localeService:Get("SettingsSystemGoldOnly"),
                tooltipText = localeService:Get("SettingsSystemGoldOnlyDescription"),
                tooltipOnButton = true,
                notCheckable = true,
                func = function()
                    -- set system
                    self.system = BidSystem.GoldOnly;
                    self:CheckSystem();

                    -- check greed mode
                    if (self.greedMode > GreedMode.DkpOnly) then
                        self.greedMode = GreedMode.DkpOnly;
                    end
                    self:CheckGreedMode();
                end
            });
            UIDropDownMenu_AddButton({
                text = localeService:Get("SettingsSystemDkpOnly"),
                tooltipTitle = localeService:Get("SettingsSystemDkpOnly"),
                tooltipText = localeService:Get("SettingsSystemDkpOnlyDescription"),
                tooltipOnButton = true,
                notCheckable = true,
                func = function()
                    -- set system
                    self.system = BidSystem.DkpOnly;
                    self:CheckSystem();

                    -- check greed mode
                    if (self.greedMode > GreedMode.DkpOnly) then
                        self.greedMode = GreedMode.DkpOnly;
                    end
                    self:CheckGreedMode();
                end
            });
            UIDropDownMenu_AddButton({
                text = localeService:Get("SettingsSystemGoldAndDkp"),
                tooltipTitle = localeService:Get("SettingsSystemGoldAndDkp"),
                tooltipText = localeService:Get("SettingsSystemGoldAndDkpDescription"),
                tooltipOnButton = true,
                notCheckable = true,
                func = function()
                    -- set system
                    self.system = BidSystem.GoldAndDkp;
                    self:CheckSystem();
                    self:CheckGreedMode();
                end
            });
        end);

        -- add greed mode drop down
        local greedModeLabel = view:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        greedModeLabel:SetPoint("TOPLEFT", 200, -40);
        greedModeLabel:SetText("Main > Second");
        local greedModeSelection = CreateFrame("Frame", nil, view, "UIDropDownMenuTemplate");
        greedModeSelection:SetPoint("TOPLEFT", 180, -54);
        greedModeSelection:SetWidth(200);
        self.greedModeSelection = greedModeSelection;
        UIDropDownMenu_Initialize(greedModeSelection, function()
            UIDropDownMenu_AddButton({
                text = localeService:Get("SettingsGreedModeDisabled"),
                tooltipTitle = localeService:Get("SettingsGreedModeDisabled"),
                tooltipText = localeService:Get("SettingsGreedModeDisabledDescription"),
                tooltipOnButton = true,
                notCheckable = true,
                func = function()
                    self.greedMode = GreedMode.Disabled;
                    self:CheckGreedMode();
                end
            }); 
            if (self.system == BidSystem.GoldAndDkp) then
                UIDropDownMenu_AddButton({
                    text = localeService:Get("SettingsGreedModeDkpOnly"),
                    tooltipTitle = localeService:Get("SettingsGreedModeDkpOnly"),
                    tooltipText = localeService:Get("SettingsGreedModeDkpOnlyDescription"),
                    tooltipOnButton = true,
                    notCheckable = true,
                    func = function()
                        self.greedMode = GreedMode.DkpOnly;
                        self:CheckGreedMode();
                    end
                }); 
                UIDropDownMenu_AddButton({
                    text = localeService:Get("SettingsGreedModeGoldAndDkp"),
                    tooltipTitle = localeService:Get("SettingsGreedModeGoldAndDkp"),
                    tooltipText = localeService:Get("SettingsGreedModeGoldAndDkpDescription"),
                    tooltipOnButton = true,
                    notCheckable = true,
                    func = function()
                        self.greedMode = GreedMode.GoldAndDkp;
                        self:CheckGreedMode();
                    end
                }); 
                UIDropDownMenu_AddButton({
                    text = localeService:Get("SettingsGreedModeMainSpecDkpOnly"),
                    tooltipTitle = localeService:Get("SettingsGreedModeMainSpecDkpOnly"),
                    tooltipText = localeService:Get("SettingsGreedModeMainSpecDkpOnlyDescription"),
                    tooltipOnButton = true,
                    notCheckable = true,
                    func = function()
                        self.greedMode = GreedMode.MainSpecDkpOnly;
                        self:CheckGreedMode();
                    end
                });
            else
                UIDropDownMenu_AddButton({
                    text = localeService:Get("SettingsGreedModeEnabled"),
                    tooltipTitle = localeService:Get("SettingsGreedModeEnabled"),
                    tooltipText = localeService:Get("SettingsGreedModeEnabledDescription"),
                    tooltipOnButton = true,
                    notCheckable = true,
                    func = function()
                        self.greedMode = GreedMode.DkpOnly;
                        self:CheckGreedMode();
                    end
                }); 
            end
        end);

        -- add gold frame
        local goldFrame = CreateFrame("Frame", nil, view);
        goldFrame:SetPoint("TOPLEFT", 240, -100);
        goldFrame:SetPoint("BOTTOMRIGHT", -16, 40);
        self.goldFrame = goldFrame;

        -- add gold title
        local goldTitle = goldFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        goldTitle:SetPoint("TOPLEFT", 0, 0);
        goldTitle:SetText("Gold");

        -- add min gold 
        local minGoldLabel = goldFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        minGoldLabel:SetTextColor(1, 1, 1);
        minGoldLabel:SetPoint("TOPLEFT", 0, -20);
        minGoldLabel:SetText(localeService:Get("SettingsGoldMinBid"));
        local minGoldEditBox = uiService:CreateNumberEditBox(goldFrame, 40);
        minGoldEditBox:SetPoint("TOPLEFT", 0, -38);
        local minGoldUnit = goldFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        minGoldUnit:SetTextColor(1, 0.85, 0);
        minGoldUnit:SetPoint("TOPLEFT", 46, -42);
        minGoldUnit:SetText(localeService:Get("Gold"));
        self.minGoldEditBox = minGoldEditBox.text;

        -- add gold step
        local goldStepLabel = goldFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        goldStepLabel:SetTextColor(1, 1, 1);
        goldStepLabel:SetPoint("TOPLEFT", 0, -70);
        goldStepLabel:SetText(localeService:Get("SettingsGoldIncrease"));
        local goldStepEditBox = uiService:CreateNumberEditBox(goldFrame, 40);
        goldStepEditBox:SetPoint("TOPLEFT", 0, -88);
        local goldStepUnit = goldFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        goldStepUnit:SetTextColor(1, 0.85, 0);
        goldStepUnit:SetPoint("TOPLEFT", 46, -92);
        goldStepUnit:SetText(localeService:Get("Gold"));
        self.goldStepEditBox = goldStepEditBox.text;

        -- add must carry gold check box
        local mustCarryGoldCheckBox = uiService:CreateCheckButton(goldFrame, 180,
                                          localeService:Get("SettingsMustCarryGold"), localeService:Get(
                                              "SettingsMustCarryGoldDescription"));
        mustCarryGoldCheckBox:SetPoint("TOPLEFT", -4, -120);
        self.mustCarryGoldCheckBox = mustCarryGoldCheckBox;

        -- add dkp frame
        local dkpFrame = CreateFrame("Frame", nil, view);
        dkpFrame:SetPoint("TOPLEFT", 16, -100);
        dkpFrame:SetPoint("BOTTOMRIGHT", view, "BOTTOMLEFT", 200, 40);
        self.dkpFrame = dkpFrame;

        -- add dkp title
        local dkpTitle = dkpFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        dkpTitle:SetPoint("TOPLEFT", 0, 0);
        dkpTitle:SetText("DKP");

        -- add dkp step
        local dkpStepLabel = dkpFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        dkpStepLabel:SetTextColor(1, 1, 1);
        dkpStepLabel:SetPoint("TOPLEFT", 0, -20);
        dkpStepLabel:SetText(localeService:Get("SettingsDkpIncrease"));
        local dkpStepEditBox = uiService:CreateNumberEditBox(dkpFrame, 40);
        dkpStepEditBox:SetPoint("TOPLEFT", 0, -38);
        local dkpStepUnit = dkpFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        dkpStepUnit:SetTextColor(0.85, 0.55, 1);
        dkpStepUnit:SetPoint("TOPLEFT", 46, -42);
        dkpStepUnit:SetText(localeService:Get("Dkp"));
        self.dkpStepEditBox = dkpStepEditBox.text;

        -- add dkp threshold
        local dkpThresholdLabel = dkpFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        dkpThresholdLabel:SetTextColor(1, 1, 1);
        dkpThresholdLabel:SetPoint("TOPLEFT", 0, -70);
        dkpThresholdLabel:SetText(localeService:Get("SettingsDkpThreshold"));
        local dkpThresholdEditBox = uiService:CreateNumberEditBox(dkpFrame, 40);
        dkpThresholdEditBox:SetPoint("TOPLEFT", 0, -88);
        dkpThresholdEditBox.text:SetScript("OnEnter", function()
            GameTooltip:SetOwner(dkpThresholdEditBox, "ANCHOR_BOTTOMLEFT");
            GameTooltip:AddLine(localeService:Get("SettingsDkpThreshold"));
            GameTooltip:AddLine(localeService:Get("SettingsDkpThresholdDescription"), 1, 1, 1, true);
            GameTooltip:Show();
        end);
        dkpThresholdEditBox.text:SetScript("OnLeave", function()
            GameTooltip:Hide();
        end);
        local dkpThresholdUnit = dkpFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        dkpThresholdUnit:SetTextColor(0.85, 0.55, 1);
        dkpThresholdUnit:SetPoint("TOPLEFT", 46, -92);
        dkpThresholdUnit:SetText(localeService:Get("Dkp"));
        self.dkpThresholdEditBox = dkpThresholdEditBox.text;

        -- add bonus dkp bid maximum
        local bonusDkpRaidMaximumLabel = dkpFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        bonusDkpRaidMaximumLabel:SetTextColor(1, 1, 1);
        bonusDkpRaidMaximumLabel:SetPoint("TOPLEFT", 0, -120);
        bonusDkpRaidMaximumLabel:SetText(localeService:Get("SettingsDkpMaxRaid"));
        local bonusDkpRaidMaximumEditBox = uiService:CreateNumberEditBox(dkpFrame, 40);
        bonusDkpRaidMaximumEditBox:SetPoint("TOPLEFT", 0, -138);
        bonusDkpRaidMaximumEditBox.text:SetScript("OnEnter", function()
            GameTooltip:SetOwner(bonusDkpRaidMaximumEditBox, "ANCHOR_BOTTOMLEFT");
            GameTooltip:AddLine(localeService:Get("SettingsDkpMaxRaid"));
            GameTooltip:AddLine(localeService:Get("SettingsDkpMaxRaidDescription"), 1, 1, 1, true);
            GameTooltip:Show();
        end);
        bonusDkpRaidMaximumEditBox.text:SetScript("OnLeave", function()
            GameTooltip:Hide();
        end);
        local bonusDkpRaidMaximumUnit = dkpFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        bonusDkpRaidMaximumUnit:SetTextColor(0.85, 0.55, 1);
        bonusDkpRaidMaximumUnit:SetPoint("TOPLEFT", 46, -142);
        bonusDkpRaidMaximumUnit:SetText(localeService:Get("BonusDkp"));
        self.bonusDkpRaidMaximumEditBox = bonusDkpRaidMaximumEditBox.text;

        -- -- add bonus dkp bid savable
        -- local bonusDkpSavableLabel = dkpFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal");
        -- bonusDkpSavableLabel:SetTextColor(1, 1, 1);
        -- bonusDkpSavableLabel:SetPoint("TOPLEFT", 0, -170);
        -- bonusDkpSavableLabel:SetText(localeService:Get("SettingsDkpMaxSavable"));
        -- local bonusDkpSavableEditBox = uiService:CreateNumberEditBox(dkpFrame, 40);
        -- bonusDkpSavableEditBox:SetPoint("TOPLEFT", 0, -188);
        -- self.bonusDkpSavableEditBox = bonusDkpSavableEditBox;
        -- local unlimitedBonusDkpCheckBox = uiService:CreateCheckButton(dkpFrame, 180,
        --                                       localeService:Get("SettingsUnlimited"), localeService:Get(
        --                                           "SettingsUnlimitedDescription"));
        -- unlimitedBonusDkpCheckBox:SetPoint("TOPLEFT", 46, -186);
        -- unlimitedBonusDkpCheckBox:SetScript("OnClick", function()
        --     if (unlimitedBonusDkpCheckBox:GetChecked()) then
        --         bonusDkpSavableEditBox:Hide();
        --         unlimitedBonusDkpCheckBox:SetPoint("TOPLEFT", -2, -186);
        --     else
        --         bonusDkpSavableEditBox:Show();
        --         unlimitedBonusDkpCheckBox:SetPoint("TOPLEFT", 46, -186);
        --         if (bonusDkpSavableEditBox.text:GetNumber() == 0) then
        --             bonusDkpSavableEditBox.text:SetNumber(20);
        --         end
        --     end
        -- end);
        -- self.unlimitedBonusDkpCheckBox = unlimitedBonusDkpCheckBox;

        -- create cancel button
        local cancelButton = uiService:CreateButton(view, CANCEL, function()
            self:Close()
        end);
        cancelButton:SetPoint("BOTTOMRIGHT", -120, 10);
        cancelButton:SetWidth(100);
        cancelButton:SetHeight(28);
        self.cancelButton = cancelButton;

        -- create next button
        local okButton = uiService:CreateButton(view, localeService:Get("Ok"), function()
            -- store values
            Settings.system = self.system;
            Settings.greedMode = self.greedMode;
            Settings.gold.minimum = self.minGoldEditBox:GetNumber();
            Settings.gold.step = self.goldStepEditBox:GetNumber();
            Settings.gold.mustCarry = self.mustCarryGoldCheckBox:GetChecked();
            Settings.dkp.step = self.dkpStepEditBox:GetNumber();
            Settings.dkp.threshold = self.dkpThresholdEditBox:GetNumber();
            Settings.dkp.bonusRaidMaximum = self.bonusDkpRaidMaximumEditBox:GetNumber();

            -- -- check bonus saving maximum
            -- if (self.unlimitedBonusDkpCheckBox:GetChecked()) then
            --     Settings.dkp.bonusSavingMaximum = 0;
            -- else
            --     Settings.dkp.bonusSavingMaximum = self.bonusDkpSavableEditBox.text:GetNumber();
            -- end

            -- trigger on update
            if (self.onUpdate ~= nil) then
                self.onUpdate();
            end

            -- close view
            self:Close();
        end);
        okButton:SetPoint("BOTTOMRIGHT", -10, 10);
        okButton:SetWidth(100);
        okButton:SetHeight(28);
    end

    -- store callbacks
    self.onUpdate = onUpdate;
    self.onClose = onClose;

    -- store system
    self.system = Settings.system;
    self:CheckSystem();

    -- store greed mode
    self.greedMode = Settings.greedMode;
    self:CheckGreedMode();

    -- fill values
    self.minGoldEditBox:SetNumber(Settings.gold.minimum);
    self.goldStepEditBox:SetNumber(Settings.gold.step);
    self.mustCarryGoldCheckBox:SetChecked(Settings.gold.mustCarry);
    self.dkpStepEditBox:SetNumber(Settings.dkp.step);
    self.dkpThresholdEditBox:SetNumber(Settings.dkp.threshold);
    self.bonusDkpRaidMaximumEditBox:SetNumber(Settings.dkp.bonusRaidMaximum);

    -- -- fill bonus saving maximum
    -- if (Settings.dkp.bonusSavingMaximum == 0) then
    --     self.unlimitedBonusDkpCheckBox:SetChecked(true);
    --     self.unlimitedBonusDkpCheckBox:SetPoint("TOPLEFT", -2, -186);
    --     self.bonusDkpSavableEditBox:Hide();
    -- else
    --     self.unlimitedBonusDkpCheckBox:SetChecked(false);
    --     self.unlimitedBonusDkpCheckBox:SetPoint("TOPLEFT", 46, -186);
    --     self.bonusDkpSavableEditBox:Show();
    --     self.bonusDkpSavableEditBox.text:SetNumber(Settings.dkp.bonusSavingMaximum);
    -- end

    -- restore position
    uiService:RestorePosition(self.view);

    -- show view
    self.view:Show();
end

--- Close view.
function SettingsView:Close()
    -- hide view
    self.view:Hide();

    -- trigger on close
    if (self.onClose ~= nil) then
        self.onClose();
    end
end

--- Check current selected system.
function SettingsView:CheckSystem()
    -- get services
    local localeService = addon:GetService("locale");
    local BidSystem = addon:GetModel("bid-system");

    -- check system
    if (self.system == BidSystem.GoldOnly) then
        UIDropDownMenu_SetText(self.systemSelection, localeService:Get("SettingsSystemGoldOnly"));
        self.dkpFrame:Hide();
        self.goldFrame:Show();
        self.goldFrame:SetPoint("TOPLEFT", 16, -100);
    elseif (self.system == BidSystem.DkpOnly) then
        UIDropDownMenu_SetText(self.systemSelection, localeService:Get("SettingsSystemDkpOnly"));
        self.dkpFrame:Show();
        self.goldFrame:Hide();
    else
        UIDropDownMenu_SetText(self.systemSelection, localeService:Get("SettingsSystemGoldAndDkp"));
        self.dkpFrame:Show();
        self.goldFrame:Show();
        self.goldFrame:SetPoint("TOPLEFT", 200, -100);
    end
end

--- Check current selected greed mode.
function SettingsView:CheckGreedMode()
    -- get services
    local localeService = addon:GetService("locale");
    local BidSystem = addon:GetModel("bid-system");
    local GreedMode = addon:GetModel("greed-mode");

    -- check system
    if (self.greedMode == GreedMode.Disabled) then
        UIDropDownMenu_SetText(self.greedModeSelection, localeService:Get("SettingsGreedModeDisabled"));
    elseif (self.greedMode == GreedMode.DkpOnly and self.system < BidSystem.GoldAndDkp) then
        UIDropDownMenu_SetText(self.greedModeSelection, localeService:Get("SettingsGreedModeEnabled"));
    elseif (self.greedMode == GreedMode.DkpOnly) then
        UIDropDownMenu_SetText(self.greedModeSelection, localeService:Get("SettingsSystemDkpOnly"));
    elseif (self.greedMode == GreedMode.GoldAndDkp) then
        UIDropDownMenu_SetText(self.greedModeSelection, localeService:Get("SettingsGreedModeGoldAndDkp"));
    else
        UIDropDownMenu_SetText(self.greedModeSelection, localeService:Get("SettingsGreedModeMainSpecDkpOnly"));
    end
end

-- register view
addon:RegisterView(SettingsView, "settings");
