--[[

@author Esperanza - Everlook/EU-Alliance
@copyright ©2021 The DKP Loot Master Authors. All Rights Reserved.

Licensed under the GNU General Public License, Version 3.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/gpl-3.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS-IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

--]]
local addon = _G.dkpLootMaster;

-- define view
StocksMainCharacterView = {};
StocksMainCharacterView.__index = StocksMainCharacterView;

--- Show view.
-- @param stock Stock with single characters to assign to main character.
-- @param callback Callback function which is triggered after successful assignment.
-- @param parent Parent view.
function StocksMainCharacterView:Show(stock, callback, parent)
    -- get services
    local uiService = addon:GetService("ui");
    local localeService = addon:GetService("locale");
    local playerService = addon:GetService("player");

    -- store stock
    self.currentStock = stock;
    self.currentStockPosition = nil;
    self.currentSelectedStock = nil;

    -- get stocks
    local stocks = {};
    for i, s in ipairs(Stocks) do
        -- check if is current player
        if (s.players[1] ~= stock.players[1]) then
            table.insert(stocks, s);
        else
            self.currentStockPosition = i;
        end
    end

    -- sort stocks by player names
    table.sort(stocks, function(a, b)
        return a.players[1] < b.players[1];
    end);

    -- get player groups
    local players = {};
    local firstPlayer = nil;
    for i, stock in ipairs(stocks) do
        -- get short name
        local shortName = playerService:GetShortName(stock.players[1]);
        local letter = string.upper(string.sub(shortName, 1, 1));

        -- check if letter exists
        if (players[letter] == nil) then
            players[letter] = {};
        end

        -- add player
        local player = {
            shortName = shortName,
            stock = stock
        };
        table.insert(players[letter], player);

        -- check if first
        if (i == 1) then
            firstPlayer = player;
        end
    end

    -- create view
    local view = uiService:CreateView("DLMStocksMainCharacter", 320, 160, localeService:Get("StocksAssignMain"));
    view:SetPoint("CENTER", parent, "CENTER", 0, 0);
    self.view = view;

    -- add select text
    local selectText = view:CreateFontString(nil, "OVERLAY", "GameFontNormal");
    selectText:SetPoint("TOPLEFT", 16, -28);
    selectText:SetPoint("BOTTOMRIGHT", view, "TOPRIGHT", -16, -80);
    selectText:SetText(localeService:Get("StocksSelectMainCharacter", playerService:GetShortName(stock.players[1])));
    selectText:SetJustifyH("LEFT");
    selectText:SetTextColor(1, 1, 1);

    -- add character drop down
    local characterSelection = CreateFrame("Frame", nil, view, "UIDropDownMenuTemplate");
    characterSelection:SetPoint("TOPLEFT", -4, -80);
    characterSelection:SetWidth(40);
    UIDropDownMenu_Initialize(characterSelection, function(dropdown, level)
        -- check if is first level
        if (level == 1) then
            -- add letters
            for letter in pairs(players) do
                UIDropDownMenu_AddButton({
                    text = letter,
                    notCheckable = true,
                    hasArrow = true,
                    value = letter,
                    keepShownOnClick = true
                }, 1);
            end
        -- check if is second level
        elseif (level == 2) then
            -- add players
            for i, player in ipairs(players[UIDROPDOWNMENU_MENU_VALUE]) do
                -- add entry
                UIDropDownMenu_AddButton({
                    text = player.shortName,
                    notCheckable = true,
                    keepShownOnClick = true,
                    func = function()
                        UIDropDownMenu_SetText(characterSelection, player.shortName);
                        self.currentSelectedStock = player.stock;
                    end
                }, 2);
            end
        end
    end);
    
    -- select first player
    if (firstPlayer ~= nil) then
        UIDropDownMenu_SetText(characterSelection, firstPlayer.shortName);
        self.currentSelectedStock = firstPlayer.stock;
    end

    -- create cancel button
    local cancelButton = uiService:CreateButton(view, CANCEL, function()
        -- hide view
        view:Hide();
        parent:Show();
    end);
    cancelButton:SetPoint("BOTTOMRIGHT", -120, 10);
    cancelButton:SetWidth(100);
    cancelButton:SetHeight(28);

    -- create ok button
    local okButton = uiService:CreateButton(view, "OK", function()
        -- check current selected stock
        if (self.currentSelectedStock ~= nil and self.currentStockPosition ~= nil) then
            -- add player name
            table.insert(self.currentSelectedStock.players, self.currentStock.players[1]);

            -- increase bonus dkp
            self.currentSelectedStock.bonusDkp = self.currentSelectedStock.bonusDkp + self.currentStock.bonusDkp;

            -- remove old stock
            table.remove(Stocks, self.currentStockPosition);
        end

        -- trigger callback
        callback();

        -- hide view
        view:Hide();
        parent:Show();
    end);
    okButton:SetPoint("BOTTOMRIGHT", -10, 10);
    okButton:SetWidth(100);
    okButton:SetHeight(28);

    parent:Hide();
    return view;
end

-- register view
addon:RegisterView(StocksMainCharacterView, "stocks-main-character");
